#!/usr/bin/python -tt
# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
from __future__ import unicode_literals

import argparse
import os
import sys

from lxml import etree as ET
from preupg.utils import OpenSCAPHelper, ProcessHelper, FileHelper

version = 1.1  # version of the preupg-diff
diff_report_name = "result_diff"
diff_report_name_xml = diff_report_name + ".xml"
diff_report_name_html = diff_report_name + ".html"
namespace = "http://checklists.nist.gov/xccdf/1.2"
# Rule result attributes to be compared for equality
rule_attrs_to_compare = ["result", "solution", "stdout", "stderr"]
parsed_opts = None


def parse_cli_opts():
    parser = argparse.ArgumentParser(
        description="preupg-diff compares multiple Preupgrade Assistant"
        " reports and shows issues that are unique to the last one."
        " For example, you have 3 reports, all of which you have already"
        " analyzed and are aware of issues raised there, and one new"
        " report.  Instead of reading through all items in the new report,"
        " this tool will go through all rule results in the last report"
        " and will hide those that have same content (e.g. solution"
        " proposal and risks) as at least one of the previous reports."
        " The trimmed report is output in XML and HTML format and named"
        " {0}.xml/html.".format(diff_report_name))
    parser.add_argument("analyzed_xml", help="One or more 'old' reports. Risks"
                        " found in these files will be filtered out from the"
                        " output.",
                        nargs='+', metavar="ANALYZED")
    parser.add_argument("new_xml", help="New XML report with possibly"
                        " new issues", metavar="NEW")
    parser.add_argument("-s", "--simple-html", help="Generate simpler HTML"
                        " than the default, without clickable buttons, just"
                        " long scrollable document.", action="store_true")
    parser.add_argument("-v", "--verbose", help="Print more detailed"
                        " information about the processing.",
                        action="store_true")
    parser.add_argument("--version", action="version", version="%(prog)s {0}"
                        .format(version))
    return parser.parse_args()


class ResultXML(object):
    def __init__(self, xml_path):
        self.tree = get_xml_tree_object(xml_path)
        self.root = self.tree.getroot()
        self.get_rules()

    def get_rules(self):
        """This method finds all the rules that has been used during the
        system preassessment and saves the following rule attributes:
        - solution .. solution/remediation text
        - result .. result string, e.g. informational, notapplicable, ..
        - stdout .. string with standard output captured during rule execution
        - stderr .. string with standard error output -||-
        - rule_tag and result_tag .. reference to the XML tag object
        """
        self.rules = {}
        self.get_rule_ids_and_solutions()
        self.get_results_and_outputs()

    def get_rule_ids_and_solutions(self):
        for rule_tag in find_subtags_recursive(self.root, "Rule"):
            rule = {}
            rule["rule_tag"] = rule_tag
            rule["solution"] = stringify_children(
                                    find_first_subtag(rule_tag,"fixtext"))
            self.rules[rule_tag.attrib["id"]] = rule

    def get_results_and_outputs(self):
        for result_tag in find_subtags_recursive(self.root, "rule-result"):
            rule_id = result_tag.attrib["idref"]
            result_text = find_first_subtag(result_tag, "result").text
            self.rules[rule_id]["result"] = result_text
            self.rules[rule_id]["result_tag"] = result_tag
            self.get_stdout_stderr(rule_id, result_tag)

    def get_stdout_stderr(self, rule_id, result_tag):
        for output_tag in find_subtags_recursive(result_tag, "check-import"):
            output_type = output_tag.attrib["import-name"]
            self.rules[rule_id][output_type] = output_tag.text

    def remove_same_result_rules(self, same_result_rules):
        for rule_id in same_result_rules.keys():
            self.remove_rule(rule_id)
            del self.rules[rule_id]

    def remove_rule(self, rule_id):
        """Remove all references to a rule from the whole XML."""
        remove_tag(self.rules[rule_id]["rule_tag"])
        remove_tag(self.rules[rule_id]["result_tag"])
        remove_tag(self.get_select_tag(rule_id))

    def get_select_tag(self, rule_id):
        for select_tag in find_subtags_recursive(self.root, "select"):
            if select_tag.attrib["idref"] == rule_id:
                return select_tag
        else:
            sys.exit("Error: Can't find select tag: {0}.".format(rule_id))

    def remove_empty_group_tags(self):
        for group_tag in find_subtags_recursive(self.root, "Group"):
            if has_no_subtags_recursive(group_tag, "Rule"):
                remove_tag(group_tag)

    def write_to_file(self):
        self.tree.write(diff_report_name_xml, pretty_print=True)


"""-----------------XML UTILS-----------------"""


def get_xml_tree_object(xml_path):
    try:
        return ET.parse(xml_path)
    except:
        sys.exit("Error: Failed to parse '{0}'.\nPreupgrade Assistant result"
                 " XML expected.".format(xml_path))


def stringify_children(tag_obj):
    """
    Return body of the given node 'tag_obj'. For example, if the node contains
    following text: 'str1<br />str2<b>str3</b>str4'
    then tag_obj.text contains just 'str1' and rest of text is thrown away.
    This function returns whole text inside the given node.
    """
    s = tag_obj.text
    if s is None:
        s = ''
    else:
        s = s.encode("UTF-8")
    for child in tag_obj:
        s += ET.tostring(child, encoding="UTF-8")
    return s


def find_subtags_recursive(tag_obj, subtag_name):
    """Returns tags with the specific name within the whole XML."""
    tag_name_with_ns = get_tag_with_ns(subtag_name)
    return tag_obj.getiterator(tag_name_with_ns)


def find_first_subtag(tag_obj, subtag_name):
    tag_name_with_ns = get_tag_with_ns(subtag_name)
    return tag_obj.find(tag_name_with_ns)


def get_tag_with_ns(tag_name):
    """All the tags in Preupgrade Assistant result XML are prepended with ns0
    namespace, which is required by python XML parsing modules when
    looking for a tag. This function returns a tag name prepended by this
    namespace and the return value can be used directly for tag searches.
    """
    return "{%s}%s" % (namespace, tag_name)


def remove_tag(tag_obj):
    parent = tag_obj.getparent()
    parent.remove(tag_obj)


def has_no_subtags_recursive(tag_obj, subtag_name):
    """Return true if tag_obj has no subtags of a particular name."""
    if sum(1 for _ in find_subtags_recursive(tag_obj, subtag_name)):
        return False
    else:
        return True


def check_files_are_readable(files):
    for file in files:
        if not FileHelper.check_file(file, "r"):
            sys.exit("Error: Can't read '{0}'.".format(file))

"""-----------------END XML UTILS-----------------"""


def get_analyzed_xml_paths(cli_paths):
    """User can enter multiple XMLs and also directory in which there are
    multiple XMLs. This function returns a list of all the XMLs.
    """
    analyzed_xml_paths = []
    for path in cli_paths:
        if os.path.isfile(path):
            analyzed_xml_paths.append(path)
        elif os.path.isdir(path):
            sys.exit("Error: Detected directory on input: '{0}'."
                     " Enter an XML file instead.".format(path))
        else:
            sys.exit("Error: Can't access '{0}'.".format(path))

    if not analyzed_xml_paths:
        sys.exit("Error: No analyzed XML found.")

    check_files_are_readable(analyzed_xml_paths)

    return analyzed_xml_paths


def load_analyzed_xmls(xml_paths):
    loaded_xmls = []
    for xml_path in xml_paths:
        loaded_xml = ResultXML(xml_path)
        loaded_xmls.append(loaded_xml)
    return loaded_xmls


def get_diff_xml(analyzed_xmls, new_xml):
    # Number of rules in the new XML
    num_new_xml_rules = len(new_xml.rules.keys())

    for analyzed_xml in analyzed_xmls:
        same_result_rules = get_rules_w_same_result(analyzed_xml.rules,
                                                    new_xml.rules)
        new_xml.remove_same_result_rules(same_result_rules)

    diff_xml = new_xml
    diff_xml.remove_empty_group_tags()

    # Number of rules left in the diff xml - subset of rules from the new XML
    num_diff_xml_rules = len(new_xml.rules.keys())

    print_difference_status(num_new_xml_rules, num_diff_xml_rules)
    return diff_xml


def get_rules_w_same_result(first_xml_rules, new_xml_rules):
    same_result_rules = {}
    for rule_id in new_xml_rules.keys():
        if rule_id not in first_xml_rules:
            continue
        # Save those new XML rules that have the same result as in the old XML
        if are_results_same(first_xml_rules[rule_id],
                            new_xml_rules[rule_id]):
            same_result_rules[rule_id] = new_xml_rules[rule_id]

    return same_result_rules


def are_results_same(first_xml_rule_attrs, new_xml_rule_attrs):
    for rule_attr in rule_attrs_to_compare:
        if rule_attr not in first_xml_rule_attrs or \
                rule_attr not in new_xml_rule_attrs:
            continue
        if first_xml_rule_attrs[rule_attr] != new_xml_rule_attrs[rule_attr]:
            return False
    return True


def print_difference_status(num_new_xml_rules, num_diff_xml_rules):
    if num_diff_xml_rules:
        if num_new_xml_rules == num_diff_xml_rules:
            verbose_print("All the rules have different results. The diff XML"
                          " is the same as the new XML.")
        else:
            verbose_print("Removed %d rules (out of %d) from the new XML."
                          "\nNote: The removed rules had exactly the same"
                          " result in one of the analyzed XMLs and in the"
                          " new XML."
                          % (num_new_xml_rules - num_diff_xml_rules,
                             num_new_xml_rules))
    else:
        verbose_print("Every rule from the new XML has exactly the same"
                      " result as in at least one of the analyzed XMLs.")


def save_diff_to_xml_and_html_file(diff_xml):
    diff_xml.write_to_file()
    fix_xml_for_simple_html()
    verbose_print("Diff XML generated: " + diff_report_name_xml)
    generate_html()
    verbose_print("Diff HTML generated: " + diff_report_name_html)


def fix_xml_for_simple_html():
    """OpenSCAP requires version of XCCDF to be 1.1 in order to correctly
    generate "simple" HTML.
    """
    if not parsed_opts.simple_html:
        return
    with open(diff_report_name_xml, 'r') as infile:
        first_line = infile.readline().replace(
            namespace, namespace.replace("1.2", "1.1"))
        rest = infile.read()
    with open(diff_report_name_xml, 'w') as outfile:
        outfile.write(first_line)
        outfile.write(rest)


def verbose_print(message):
    if parsed_opts.verbose:
        print(message)


def generate_html():
    if not os.path.exists(diff_report_name_xml):
        sys.exit("Error: HTML generation failed: source {0} not found."
                 .format(diff_report_name_xml))
    cmd = OpenSCAPHelper.build_generate_command(diff_report_name_xml,
                                                diff_report_name_html,
                                                parsed_opts.simple_html)
    log = "html.log"  # log for text printed during html generation
    ret_val = ProcessHelper.run_subprocess(cmd, print_output=True,
                                           output=log)
    if ret_val != 0:
        sys.exit("Error: HTML generation failed. See {0} for details."
                 .format(log))
    else:  # no error when generating html - safe to remove the log
        os.remove(log)


def run():
    """The main entry point of the preupg-diff tool"""
    global parsed_opts
    parsed_opts = parse_cli_opts()
    analyzed_xml_paths = get_analyzed_xml_paths(parsed_opts.analyzed_xml)
    analyzed_xmls = load_analyzed_xmls(analyzed_xml_paths)
    new_xml = ResultXML(parsed_opts.new_xml)
    diff_xml = get_diff_xml(analyzed_xmls, new_xml)
    save_diff_to_xml_and_html_file(diff_xml)



from __future__ import unicode_literals, print_function
import unittest
import tempfile
import os
import shutil
import ConfigParser

try:
    import base
except ImportError:
    import tests.base as base

from preupg import settings
from preupg.creator import ui_helper
from preupg.creator.ui_helper import UIHelper
from preupg.utils import FileHelper


def load_file(filename):
    try:
        lines = FileHelper.get_file_content(filename, "rb", True)
        lines = [x.strip() for x in lines]
    except IOError:
        assert False
    return lines


class TestCreator(base.TestCase):

    tempdir = ""
    upgrade_dir = ""
    puh = None
    group_name = "foobar_group"
    content_name = settings.module_ini
    content_dict = {}

    class Identity(base.MockFunction):
        def __init__(self, identity):
            self.identity = identity

        def __call__(self, *args, **kwargs):
            return self.identity

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.upgrade_dir = os.path.join(self.tempdir, "FOOBAR6_7")
        os.makedirs(self.upgrade_dir)
        self.puh = UIHelper(self.upgrade_dir)
        self.puh._group_name = self.group_name
        self.puh._content_name = self.content_name
        self.puh.check_script = True
        self.puh.solution_file = True
        self.content_dict['content_title'] = "foobar_test_title"
        self.content_dict['content_description'] = "Foobar content test description"
        self.puh.content_dict = self.content_dict
        self.puh.content_path = os.path.join(self.puh.get_group_name(), self.puh.get_content_name())
        if os.path.exists(self.puh.get_content_path()):
            shutil.rmtree(self.puh.get_content_path())
        os.makedirs(self.puh.get_content_path())

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    @base.mock(UIHelper, "properties_ini_exists", Identity(True))
    def test_content_ini(self):
        self.puh.script_type = "sh"
        self.puh.prepare_content_env()
        self.puh.create_final_content()
        content_ini = os.path.join(self.puh.get_content_path(),
                                   self.puh.get_content_ini_file())
        self.assertEqual(content_ini, os.path.join(self.upgrade_dir,
                                                   self.group_name,
                                                   self.content_name,
                                                   self.content_name))
        lines = load_file(content_ini)
        expected_ini = ['[preupgrade]',
                        'content_description = Foobar content test description',
                        'content_title = foobar_test_title',
                        '']
        self.assertEqual(expected_ini.sort(), lines.sort())

    @base.mock(UIHelper, "properties_ini_exists", Identity(True))
    def test_group_ini(self):
        self.puh.script_type = "sh"
        self.puh.prepare_content_env()
        self.puh.create_final_content()
        group_ini = os.path.join(self.puh.get_upgrade_path(),
                                 self.puh.get_group_name(), 'group.ini')
        self.assertEqual(group_ini, os.path.join(self.upgrade_dir,
                                                 self.group_name,
                                                 'group.ini'))
        lines = load_file(group_ini)
        expected_group_ini = ['[preupgrade]',
                              'group_title = Title for foobar_group',
                              '']
        self.assertEqual(lines, expected_group_ini)

    @base.mock(UIHelper, "properties_ini_exists", Identity(True))
    def test_bash_check_script(self):
        self.puh.script_type = "sh"
        self.puh.prepare_content_env()
        self.puh.create_final_content()
        check_script = os.path.join(self.puh.get_content_path(), self.puh.get_check_script())
        lines = load_file(check_script)
        exp_script = ['#!/bin/bash',
                      '',
                      '. /usr/share/preupgrade/common.sh',
                      '',
                      '#END GENERATED SECTION',
                      '',
                      "### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'.",
                      '']
        self.assertEqual(lines, exp_script)

    @base.mock(UIHelper, "properties_ini_exists", Identity(True))
    def test_python_check_script(self):
        self.puh.script_type = "python"
        self.puh.prepare_content_env()
        self.puh.create_final_content()
        check_script = os.path.join(self.puh.get_content_path(), self.puh.get_check_script())
        lines = load_file(check_script)
        exp_script = ['#!/usr/bin/python',
                      '# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-', '',
                      'import sys',
                      'import os', '',
                      'from preupg.script_api import *', '',
                      '#END GENERATED SECTION', '',
                      "### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'."]
        self.assertTrue(exp_script, lines)

    @base.mock(UIHelper, "properties_ini_exists", Identity(False))
    def test_create_properties_ini(self):
        """ Create valid properties.ini file """
        ini_content = ["[preupgrade-assistant-modules]",
                       "src_major_version = 6",
                       "dst_major_version = 7",
                       ""]

        ui = UIHelper()
        ui.src_version = 6
        ui.dst_version = 7

        tmp_file_name = tempfile.mkstemp()[1]
        ui.properties_ini_path = tmp_file_name
        ui.create_properties_ini()
        tmp_content = load_file(tmp_file_name)
        self.assertEqual(tmp_content, ini_content)
        os.remove(tmp_file_name)

    @base.mock(UIHelper, "check_path", Identity(True))
    def test_specify_upgrade_path(self):
        """Normal pass"""
        self.assertTrue(self.puh.specify_upgrade_path())

    def test_is_valid_string(self):
        self.assertTrue(UIHelper.is_valid_string('text'))
        self.assertTrue(UIHelper.is_valid_string(u'text'))

        self.assertFalse(UIHelper.is_valid_string(""))
        self.assertFalse(UIHelper.is_valid_string(None))
        self.assertFalse(UIHelper.is_valid_string(0))

    @base.mock(ui_helper, "get_user_input", Identity('6'))
    def test_ask_about_version_number(self):
        self.assertEqual(UIHelper.ask_for_string(
            "msg", "err_msg"), '6')

    @base.mock(ui_helper, "get_user_input", Identity(''))
    def test_ask_about_wrong_version(self):
        self.assertEqual(UIHelper.ask_for_string(
            "msg", "err_msg"), False)

    def test_write_config_to_file(self):
        """ Test creation and content in config file """
        config = ConfigParser.RawConfigParser()
        section = "section"
        config.add_section(section)
        config.set(section, 'option', 'value')

        tmp_file_name = tempfile.mkstemp()[1]
        UIHelper.write_config_to_file(tmp_file_name, config)
        tmp_content = load_file(tmp_file_name)
        expected_content = ['[section]', 'option = value', '']
        self.assertEqual(tmp_content, expected_content)
        os.remove(tmp_file_name)

    def test_write_config_to_nofile(self):
        """ Write config to non-existing file path """
        self.assertRaises(IOError, UIHelper.write_config_to_file, '', None)


def suite():
    loader = unittest.TestLoader()
    unit_suite = unittest.TestSuite()
    unit_suite.addTest(loader.loadTestsFromTestCase(TestCreator))
    return unit_suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



class DummyConfKickstart(object):
    """
    Dummy conf class for Conf

    use it like this:
    conf = Conf(DummyConf(id=123, skip_common=True))
    """
    def __init__(self, **kwargs):
        self.settings = kwargs

    def __getattr__(self, name):
        try:
            return self.settings.get(name)
        except AttributeError:
            return object.__getattribute__(self, name)


class ConfKickstart(object):
    """
    configuration of preupgrade assistant

    merged values from CLI and settings.py
    """

    def __init__(self, *args):
        """
        *args - list of objects with settings attached as
                attributes to these objects
        priority:
            args[0] > args[1] > ...
        """
        self.settings = list(args)

    def insert(self, pos, settings):
        self.settings.insert(pos, settings)

    def __getattr__(self, name):
        for arg in self.settings:
            try:
                value = getattr(arg, name)
            except AttributeError:
                continue
            if value is not None and value != "":
                return value
            else:
                continue
        return None



# -*- coding: utf-8 -*-

from django import forms
from preupg.ui.report.models import TestResult
from django.http import QueryDict

class GenericStateFilterForm(forms.Form):
    """ filter test results by their state """

    def __init__(self, *args, **kwargs):
        super(GenericStateFilterForm, self).__init__(*args, **kwargs)

        states = TestResult.TEST_STATES.get_display_mapping()

        self.fields['state'] = forms.MultipleChoiceField(choices=states, required=False)
        #if initial_list:
        #    qd = QueryDict('state=a', mutable=True)
        #    qd.setlist('state', initial_list)
        #    print qd
        #    print qd.getlist('state')
        #    self.fields['state'].initial = qd



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import datetime
import re
import subprocess
import fnmatch
import os
import sys
import shutil
import tempfile
import mimetypes
import platform
import codecs

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from preupg import settings
from preupg.logger import log_message, logging, logger, logger_debug

from os import path, access, W_OK, R_OK, X_OK

try:
    from hashlib import sha1
except ImportError:
    from sha import sha as sha1


def get_current_time():
    return datetime.datetime.now().strftime("%y%m%d%H%M%S")


class MessageHelper(object):

    @staticmethod
    def print_error_msg(title="", msg="", level=' ERROR '):
        """Function prints a ERROR or WARNING messages"""
        number = 10
        print('\n')
        print('*' * number + level + '*' * number)
        print(title, ''.join(msg))

    @staticmethod
    def get_message(title="", default_yes=True, message="Do you want to continue?", prompt=None):
        """
        Function asks for input from user

        :param title: Title of the message
        :param message: Message text
        :param default_yes: If the deafult values is YES
        :return: y or n
        """
        yes = ['yes', 'y', 'Y', 'Yes']
        yesno = yes + ['no', 'n', 'N', 'No']
        print (title)
        if prompt is not None:
            print (message + ' ' + prompt)
        else:
            print (message)
        while True:
            if int(sys.version_info[0]) == 2:
                choice = raw_input()
            else:
                choice = input()
            if not choice:
                if default_yes:
                    return "yes"
            if prompt and choice not in yesno:
                print ('You have to choose one of y/n.')
            else:
                return choice


class FileHelper(object):

    @staticmethod
    def check_file(fp, mode):
        """
        Check if file exists and has set right mode

        mode can be in string format as for function open (available letters: wrax)
        or int number (in that case prefered are os constants W_OK, R_OK, X_OK)
        (letter 'a' has same signification as 'w', is here due to compatibility
        with open mode)
        """
        intern_mode = 0
        if isinstance(mode, int):
            intern_mode = mode
        else:
            if 'w' in mode or 'a' in mode:
                intern_mode += W_OK
            if 'r' in mode:
                intern_mode += R_OK
            if 'x' in mode:
                intern_mode += X_OK
        if path.exists(fp):
            if path.isfile(fp):
                if access(fp, intern_mode):
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False

    @staticmethod
    def check_xml(xml_file):
        """
        Check XML

        return False if xml file is not okay or raise IOError if perms are
        not okay; use python-magic to check the file if module is available
        """
        if os.path.isfile(xml_file):
            if not os.access(xml_file, os.R_OK):
                log_message("The file is not readable." % xml_file, level=logging.ERROR)
                raise IOError("The %s file is not readable." % xml_file)
        else:
            log_message("%s is not a file" % xml_file, level=logging.ERROR)
            raise IOError("%s is not a file." % xml_file)
        raw_test = False
        is_valid = False
        try:
            import magic
        except ImportError:
            raw_test = True
        else:
            try:
                xml_file_magic = magic.from_file(xml_file, mime=True)
            except AttributeError:
                raw_test = True
            else:
                is_valid = xml_file_magic == 'application/xml'
        if raw_test:
            is_valid = xml_file.endswith(".xml")
        if is_valid:
            return xml_file
        else:
            log_message("The provided file is not a valid XML file",
                        level=logging.ERROR)
            raise RuntimeError("The provided file is not a valid XML file")

    @staticmethod
    def get_interpreter(filename, verbose=False):
        """
        The function returns interpreter

        Checks extension of script and first line of script
        """
        script_types = {'/bin/bash': '.sh',
                        '/usr/bin/python': '.py',
                        '/usr/bin/perl': '.pl'}
        inter = list(k for k, v in iter(script_types.items())
                     if filename.endswith(v))
        content = FileHelper.get_file_content(filename, 'rb')
        if inter and content.startswith('#!' + inter[0]):
            return inter
        else:
            if verbose:
                log_message("Problem with getting an interpreter",
                            level=logging.ERROR)
            return None

    @staticmethod
    def get_file_content(full_path, perms, method=False, decode_flag=True):
        """
        shortcut for returning content of file

        open(...).read()...
        if method is False then file is read by function read
        if method is True then file is read by function readlines
        When decode_flag is True, read string is decoded to unicode. Otherwise
        only read. (Some libraries request non-unicode strings - as
        ElementTree)
        """

        # data must be init due to possible troubles with binary data
        data = None
        if decode_flag:
            f = codecs.open(full_path, perms, settings.defenc)
        else:
            f = open(full_path, perms)
        try:
            data = f.read() if not method else f.readlines()
        finally:
            f.close()
        if data is None:
            raise ValueError("You are tring decode binary data to unicode: %s"
                             % path)
        return data

    @staticmethod
    def write_to_file(full_path, perms, data, encode_flag=True):
        """
        shortcut for write of data to file:

        open(...).write()...
        data can be string or list of strings

        data contains unicode string(s) in most cases, so we encode them
        to system default encoding before write. When you use encoded strings,
        set encode_flag to False to suppress second encodiding process.
        """
        f = open(full_path, perms)
        try:
            if isinstance(data, list):
                if encode_flag is True:
                    data = [line.encode(settings.defenc) for line in data]
                f.writelines(data)
            else:
                # TODO: May we should print warn w
                if encode_flag is True and isinstance(data, unicode):
                    f.write(data.encode(settings.defenc))
                else:
                    f.write(data)
        finally:
            f.close()

    @staticmethod
    def remove_home_issues():
        """
        Function removes /home rows from specific files

        :return:
        """
        files = [os.path.join(settings.cache_dir, settings.common_name, 'allmyfiles.log'),
                 os.path.join(settings.KS_DIR, 'untrackeduser')]
        for f in files:
            try:
                lines = FileHelper.get_file_content(f, 'rb', method=True)
                lines = [l for l in lines if not l.startswith('/home')]
                FileHelper.write_to_file(f, 'wb', lines)
            except IOError:
                pass

    @staticmethod
    def check_executable(file_name):
        """
        The function checks whether script is executable.
        If not then ERROR message arise
        """
        if not os.access(file_name, os.X_OK):
            MessageHelper.print_error_msg(title="The %s file is not executable" % file_name)

    @staticmethod
    def get_script_type(file_name):
        """
        The function returns type of check_script.
        If it's not any script then return just txt
        """
        mime_type = mimetypes.guess_type(file_name)[0]
        if mime_type is None:
            # try get mime type with shebang
            line = FileHelper.get_file_content(file_name, "rb", True)[0]
            if line.startswith("#!"):
                if re.search(r"\bpython[0-9.-]*\b", line):
                    return 'python'
                if re.search(r"\bperl[0-9.-]*\b", line):
                    return 'perl'
                if re.search(r"\bcsh\b", line):
                    return 'csh'
                if re.search(r"\b(bash|sh)\b", line):
                    return 'sh'
        file_types = {'text/x-python': 'python',
                      'application/x-csh': 'csh',
                      'application/x-sh': 'sh',
                      'application/x-perl': 'perl',
                      'text/plain': 'txt',
                      'text/x-sh': 'sh',
                      None: 'txt',
                      }
        return file_types[mime_type]

    @staticmethod
    def get_list_executable_files_in_dir(dir_name):
        """
        The function returns list of executable files in directory.

        :param dir_name: Dir name where executable files are searched
        :return: List of files or empty list
        """
        found_scripts = []
        for (root_path, dirs, files) in os.walk(dir_name):
            for f in files:
                file_name = os.path.join(root_path, f)
                if os.path.exists(file_name) and FileHelper.check_file(file_name, "x") is True:
                    found_scripts.append(file_name)
        return found_scripts


class DirHelper(object):

    @staticmethod
    def check_or_create_temp_dir(temp_dir, mode=None):
        """Check if provided temp dir is valid."""
        if os.path.isdir(temp_dir):
            if not os.access(temp_dir, os.W_OK):
                log_message("The %s directory is not writable." % temp_dir, level=logging.ERROR)
                raise IOError("The %s directory is not writable." % temp_dir)
        else:
            os.makedirs(temp_dir)
        if mode:
            os.chmod(temp_dir, mode)
        return temp_dir

    @staticmethod
    def create_dest_dir(full_path):
        n = datetime.datetime.now()
        stamp = n.strftime("%y%m%d%H%M%S%f")
        if full_path.endswith('/'):
            destdir = full_path[:-1] + stamp
        else:
            destdir = full_path + stamp
        os.makedirs(destdir)
        return destdir

    @staticmethod
    def clean_directory(dir_name, pattern):
        """
        Function deleted specific files in dir_name

        :param dir_name: Dirname where the files are deleted
        :param pattern: What files with specific pattern are deleted
        :return:
        """
        for root, dummy_dirs, files in os.walk(dir_name):
            for f in files:
                if fnmatch.fnmatch(f, pattern):
                    os.unlink(os.path.join(root, f))


class ProcessHelper(object):

    @staticmethod
    def run_subprocess(cmd, output=None, print_output=False, shell=False, function=None):
        """wrapper for Popen"""
        sp = subprocess.Popen(cmd,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
                              shell=shell,
                              bufsize=1)
        stdout = str() # FIXME should't be this bytes()?
        for stdout_data in iter(sp.stdout.readline, b''):
            # communicate() method buffers everything in memory, we will read stdout directly
            stdout += stdout_data
            if function is None:
                if print_output:
                    print (stdout_data, end="")
                else:
                    pass
            else:
                # I don't know what functions can come here, however
                # it's not common so put only unicode data here again.
                # Should be always raw data so we don't need test stdout_data
                # on type
                function(stdout_data.decode(settings.defenc))
        sp.communicate()

        if output is not None:
            # raw data, so without encoding
            FileHelper.write_to_file(output, "wb", stdout, False)
        return sp.returncode


class SystemIdentification(object):

    @staticmethod
    def get_arch():
        return platform.machine()

    @staticmethod
    def get_convertors():
        """Function returns list of supported convertors"""
        return settings.text_converters.keys()


    @staticmethod
    def get_variant():
        """Function return a variant"""
        redhat_release = FileHelper.get_file_content("/etc/redhat-release", "rb")
        if redhat_release.startswith('Fedora'):
            return None
        try:
            rel = redhat_release.split()
            return rel[4]
        except IndexError:
            return None


class TarballHelper(object):

    @staticmethod
    def _get_tarball_name(result_file, time):
        return result_file.format(time)

    @staticmethod
    def _get_tarball_result_path(root_dir, filename):
        return os.path.join(root_dir, filename)

    @staticmethod
    def tarball_result_dir(result_file, verbose):
        """
        pack results to tarball
        """
        current_dir = os.getcwd()
        tar_binary = "/bin/tar"
        current_time = get_current_time()
        cmd_pack = "-czvf"
        cmd = [tar_binary]
        # numeric UIDs and GIDs are used, ACLs are enabled, SELinux is enabled
        tar_options = ["--numeric-owner", "--acls", "--selinux"]

        # used for packing directories into tarball
        os.chdir('/root')
        tarball_dir = TarballHelper._get_tarball_name(result_file, current_time)
        tarball_name = tarball_dir + '.tar.gz'
        bkp_tar_dir = os.path.join('/root', tarball_dir)

        if not os.path.exists(bkp_tar_dir):
            os.makedirs(bkp_tar_dir)
        for dir_to_pack in settings.preupgrade_dirs:
            shutil.copytree(os.path.join(settings.assessment_results_dir,
                                         dir_to_pack),
                            os.path.join(bkp_tar_dir, dir_to_pack),
                            symlinks=True)
        files_to_copy = [settings.PREUPG_README]
        for _, _, files in os.walk(settings.assessment_results_dir):
            for f in files:
                if f.startswith("result"):
                    files_to_copy.append(f)
        for f in files_to_copy:
            shutil.copyfile(os.path.join(settings.assessment_results_dir, f),
                            os.path.join(bkp_tar_dir, f))
        tarball = TarballHelper._get_tarball_result_path(
            settings.assessment_results_dir, tarball_name)
        cmd.append(cmd_pack)
        cmd.append(tarball)
        cmd.append(tarball_dir)

        cmd.extend(tar_options)
        ProcessHelper.run_subprocess(cmd, print_output=verbose)
        shutil.rmtree(bkp_tar_dir)
        shutil.copy(tarball, settings.tarball_result_dir + "/")
        os.chdir(current_dir)

        return os.path.join(settings.tarball_result_dir, tarball_name)

    @staticmethod
    def get_latest_tarball(result_dir):
        """Returns full tarball path"""
        if not os.path.isdir(result_dir):
            return None
        full_tarball_name = sorted([f for f in os.listdir(result_dir)], reverse=True)
        # Return the latest tarball
        if full_tarball_name:
            return os.path.join(result_dir, full_tarball_name[0])
        else:
            return None


class ConfigHelper(object):
    @staticmethod
    def get_preupg_config_file(full_path, key, section="preupgrade-assistant"):
        if not os.path.exists(full_path):
            return None

        config = configparser.RawConfigParser()
        config.read(full_path)
        if config.has_section(section):
            if config.has_option(section, key):
                return config.get(section, key)

    @staticmethod
    def config_has_section(config_path, section):
        if not os.path.exists(config_path):
            return False
        config = configparser.RawConfigParser()
        config.read(config_path)
        return True if config.has_section(section) else False


class ConfigFilesHelper(object):
    @staticmethod
    def check_cleanconf_dir(result_dir, cleanconf):
        # Check if configuration file exists in /root/preupgrade/cleanconf directory
        clean_conf_name = os.path.join(result_dir, cleanconf)
        if os.path.exists(clean_conf_name):
            message = "The '%s' configuration file already exists in the '%s' directory"
            logger.info(message % (clean_conf_name, os.path.dirname(clean_conf_name)))
            # Check if configuration file exists in dirtyconf
            # If so delete them.
            return True
        return False

    @staticmethod
    def check_dirtyconf_dir(dirtyconf, filename):
        # Check if configuration file exists in /root/preupgrade/dirtyconf directory
        # If not return real path of configuration file. Not a symlink.
        full_path = filename
        # Copy filename to dirtyconf directory
        # Check if file is a symlink or real path.
        if os.path.islink(full_path):
            full_path = os.path.realpath(full_path)
        # Check if configuration file exists in dirtyconf directory
        if os.path.exists(dirtyconf):
            logger.info("The '%s' file already exists in the dirtyconf directory", dirtyconf)
            return False
        # Check whether dirtyconf directory with dirname(filename) exists
        if not os.path.exists(os.path.dirname(dirtyconf)):
            os.makedirs(os.path.dirname(dirtyconf))
        return full_path

    @staticmethod
    def copy_modified_config_files(result_dir):
        """
        Function copies all modified files to dirtyconf directory.

        (files which are not mentioned in cleanconf directory)
        """
        etc_va_log = os.path.join(settings.cache_dir, settings.common_name, "rpm_etc_Va.log")
        try:
            lines = FileHelper.get_file_content(etc_va_log, "rb", method=True)
        except IOError:
            raise IOError("Error: File that lists modified configuration files"
                          "'%s' is missing.\n" % etc_va_log)
        dirty_conf = os.path.join(result_dir, settings.dirty_conf_dir)
        clean_conf = os.path.join(result_dir, settings.clean_conf_dir)
        # Go through all changed config files
        for line in lines:
            try:
                (opts, flags, filename) = line.strip().split()
                if opts.strip() == 'missing':
                    continue
            except ValueError:
                return
            logger_debug.debug("The '%s' file name to copy.", filename)
            new_filename = filename[1:]
            # Check whether config file exists in cleanconf directory
            cleanconf_file_name = os.path.join(clean_conf, new_filename)
            dirtyconf_file_name = os.path.join(dirty_conf, new_filename)
            # Check if config file does not exist in cleanconf directory
            if ConfigFilesHelper.check_cleanconf_dir(result_dir, cleanconf_file_name):
                if os.path.exists(dirtyconf_file_name):
                    log_message("The %s file exist in the %s directory" % (new_filename, dirty_conf), logging.DEBUG)
                    os.unlink(dirtyconf_file_name)
                continue
            # Check if config file does not exists in dirtyconf directory
            check = ConfigFilesHelper.check_dirtyconf_dir(dirtyconf_file_name, filename)
            if check:
                try:
                    shutil.copyfile(check, dirtyconf_file_name)
                except IOError:
                    sys.stderr.write("Warning: Could not copy '%s' to '%s'.\n"
                                     % (check,
                                        os.path.dirname(dirtyconf_file_name)))
                    pass


class PostupgradeHelper(object):

    @staticmethod
    def get_all_postupgrade_files(dummy_verbose, dir_name):
        """Function gets all postupgrade files from dir_name"""
        postupg_scripts = []
        for root, dummy_sub_dirs, files in os.walk(dir_name):
            # find all files in this directory
            postupg_scripts.extend([os.path.join(root, x) for x in files])
        if not postupg_scripts:
            log_message("No postupgrade scripts available")
        return postupg_scripts

    @staticmethod
    def get_hash_file(filename, hasher):
        """Function gets a hash from file"""
        content = FileHelper.get_file_content(filename, "rb", False, False)
        hasher.update(b'preupgrade-assistant' + content)
        return hasher.hexdigest()

    @staticmethod
    def postupgrade_scripts(verbose, dirname):
        """
        The function runs postupgrade directory

        If dir does not exists the report and return
        """
        if not os.path.exists(dirname):
            log_message('There is no any %s directory' % settings.postupgrade_dir,
                        level=logging.WARNING)
            return

        postupg_scripts = PostupgradeHelper.get_all_postupgrade_files(verbose, dirname)
        if not postupg_scripts:
            return

        #max_length = max(list([len(x) for x in postupg_scripts]))

        log_message('Running postupgrade scripts:')
        for scr in sorted(postupg_scripts):
            interpreter = FileHelper.get_interpreter(scr, verbose=verbose)
            if interpreter is None:
                continue
            log_message('Executing script %s' % scr)
            cmd = "{0} {1}".format(interpreter, scr)
            ProcessHelper.run_subprocess(cmd, print_output=False, shell=True)
            log_message("Executing script %s ...done" % scr)

    @staticmethod
    def get_hashes(filename):
        """Function gets all hashes from a filename"""
        if not os.path.exists(filename):
            return None
        hashed_file = FileHelper.get_file_content(filename, "rb").split()
        hashed_file = [x for x in hashed_file if "hashed_file" not in x]
        return hashed_file

    @staticmethod
    def hash_postupgrade_file(verbose, dirname, check=False):
        """
        The function creates hash file over all scripts in postupgrade.d directory.

        In case of remediation it checks whether checksums are different and
        print what scripts were changed.
        """
        if not os.path.exists(dirname):
            message = 'The %s directory does not exist for creating checksum file'
            log_message(message, settings.postupgrade_dir, level=logging.ERROR)
            return

        postupg_scripts = PostupgradeHelper.get_all_postupgrade_files(verbose, dirname)
        if not postupg_scripts:
            return

        filename = settings.base_hashed_file
        if check:
            filename = settings.base_hashed_file + "_new"
        lines = []
        for post_name in postupg_scripts:
            lines.append(post_name + "=" + PostupgradeHelper.get_hash_file(post_name, sha1())+"\n")

        full_path_name = os.path.join(dirname, filename)
        FileHelper.write_to_file(full_path_name, "wb", lines)

        if check:
            hashed_file = PostupgradeHelper.get_hashes(os.path.join(dirname, settings.base_hashed_file))
            if hashed_file is None:
                message = 'The Hashed_file is missing. The postupgrade scripts will not be executed'
                log_message(message, level=logging.WARNING)
                return False
            hashed_file_new = PostupgradeHelper.get_hashes(full_path_name)
            different_hashes = list(set(hashed_file).difference(set(hashed_file_new)))
            for file_name in [settings.base_hashed_file, filename]:
                os.remove(os.path.join(dirname, file_name))
            if different_hashes or len(different_hashes) > 0:
                message = 'The checksums are different in these postupgrade scripts: %s'
                log_message(message % different_hashes, level=logging.WARNING)
                return False
        return True

    @staticmethod
    def special_postupgrade_scripts(result_dir):
        """
        The function copies a special postupgrade.d scripts.

        postupgrade_dict is a dictionary with old and new files
        Files are copied from
                /usr/share/preupgrade/postupgrade.d/<key> directory
        to
                /root/preupgrade/postupgrade.d/<val> directory
        with the corresponding names
        mentioned in postupgrade.d directory.
        """
        postupgrade_dict = {"copy_clean_conf.sh": "z_copy_clean_conf.sh",
                            "postupgrade_hooks.sh": "postupgrade_hooks.sh"}

        for key, val in iter(postupgrade_dict.items()):
            source_file = os.path.join(settings.data_dir,
                                       settings.postupgrade_dir, key)
            if os.path.exists(source_file):
                destination_file = os.path.join(result_dir,
                                                settings.postupgrade_dir,
                                                val)
                shutil.copy(source_file, destination_file)


class OpenSCAPHelper(object):

    def __init__(self, result_dir, result_name, xml_result_name, html_result_name, content):
        self.binary = [settings.openscap_binary]
        self.result_dir = result_dir
        self.xml_result_name = xml_result_name
        self.html_result_name = html_result_name
        self.result_name = result_name
        self.content = content

    def update_variables(self, result_dir, result_name, xml_result_name, html_result_name, content):
        self.result_dir = result_dir
        self.xml_result_name = xml_result_name
        self.html_result_name = html_result_name
        self.result_name = result_name
        self.content = content

    @staticmethod
    def get_oscap_version():
        """
        Get version of the OpenSCAP.

        Return version of the openscap as dict with the 'major', 'minor' and
        'patch' keys.

        Raise OSError when the oscap utility is not installed.
        """
        oscap_version = {"major" : -1, "minor" :  -1, "patch" : -1}
        def _process_oscap_output(oscap_msg):
            if oscap_version["major"] >= 0:
                # This is confusing check, but the ProcessHelper.run_subprocess
                # processes each line of stdout separately and oscap version
                # appears only on the first line. So in case of:
                # the first line
                #    the oscap_version["major"] == -1
                # other lines
                #    the oscap_version["major"] >= 0
                return
            # Processing of the first line
            rmatch = re.match(r".*\s([0-9]+)\.([0-9]+)\.([0-9]+)\s$", oscap_msg)
            if rmatch is None:
                return
            oscap_version["major"] = int(rmatch.group(1))
            oscap_version["minor"] = int(rmatch.group(2))
            oscap_version["patch"] = int(rmatch.group(3))
        ProcessHelper.run_subprocess([settings.openscap_binary, "version"],
                print_output=False, function=_process_oscap_output)
        return oscap_version

    @staticmethod
    def is_oscap_equal_or_greater(major, minor, patch):
        "Return True when version of the OpenSCAP is eg then given one."
        oscap_version = OpenSCAPHelper.get_oscap_version()
        if (oscap_version["major"] >= major
                and oscap_version["minor"] >= minor
                and oscap_version["patch"] >= patch):
            return True
        return False

    @staticmethod
    def get_xsl_stylesheet(old_style=False):
        """Return full XSL stylesheet path"""
        if old_style:
            subfolder = "simple"
        else:
            subfolder = "complex"
        return os.path.join(settings.data_dir, "report", subfolder,
                            settings.xsl_sheet)

    @staticmethod
    def build_generate_command(xml_file, html_file, old_style=False):
        """Function builds a command for generating results"""
        command = [settings.openscap_binary]
        command.extend(['xccdf', 'generate', 'custom'])
        command.extend(["--stylesheet",
                        OpenSCAPHelper.get_xsl_stylesheet(old_style)])
        command.extend(("--output", html_file))
        command.append(FileHelper.check_xml(xml_file))
        return command

    def build_command(self):
        """create command from configuration"""
        command_eval = ['xccdf', 'eval']
        result_file = self.get_default_xml_result_path()
        command = [settings.openscap_binary]
        command.extend(command_eval)
        command.append('--progress')
        command.extend(('--profile', settings.profile))

        command.extend(("--results", result_file))
        command.append(FileHelper.check_xml(self.content))
        return command

    def get_default_xml_result_path(self):
        """Returns full XML result path"""
        return os.path.join(self.result_dir, self.xml_result_name)

    def get_default_html_result_path(self):
        """Returns full HTML result path"""
        return os.path.join(self.result_dir, self.html_result_name)

    def get_default_txt_result_path(self):
        """
        Function returns default txt result path based on result_dir

        :return: default txt result path
        """
        return os.path.join(self.result_dir, self.result_name + ".txt")

    def run_generate(self, xml_file, html_file, old_style=False):
        """
        The function generates result.html file from result.xml file
        which was modified by preupgrade assistant
        """
        cmd = self.build_generate_command(xml_file, html_file, old_style=old_style)
        out_path = tempfile.mktemp(prefix='prefix.run_generate.')
        ret_val = ProcessHelper.run_subprocess(cmd, print_output=False, output=out_path)
        lines = FileHelper.get_file_content(out_path, 'r', method=True)
        logger.debug('%s', '\n'.join(lines))
        os.remove(out_path)
        return ret_val


class ModuleSetUtils(object):

    @staticmethod
    def get_config_key_value(config_path, key, section):
        """
        Checks for key in section if key exists, returns it else raise
        an exception

        @param {string} config_path
        @param {string} key
        @param {string} section
        @return {string} - key value
        @throws {EnvironmentError} - if key of section doesn't exist
        """
        if not ConfigHelper.config_has_section(config_path, section):
            raise EnvironmentError("Section {0} is missing inside {1} file is"
                                   .format(section, config_path))
        key_value = ConfigHelper.get_preupg_config_file(config_path,
                                                        key, section)
        if not key_value:
            raise EnvironmentError("Key {0} inside {1} section\n"
                                   "is missing in {2}"
                                   .format(key, section, config_path))
        return key_value

    @staticmethod
    def get_module_set_os_versions(module_set_path):
        """
        Check if properties.ini file exists in dir_name and also
        validate the file content. From the content gets src/dst version of OS
        and return them as list.

        @param {string} module_set_path - eg. /root/preupgrage/modul_dir
        @return {list}
        @throws {EnvironmentError} - when file doesn't exist or content is
                                     incorrect
        """
        src_version_key = "src_major_version"
        dst_version_key = "dst_major_version"
        section_name = "preupgrade-assistant-modules"
        if not os.path.isdir(module_set_path):
            module_set_path = os.path.join(settings.source_dir,
                                           module_set_path)
        properties_file = os.path.join(module_set_path,
                                       settings.properties_ini)
        if not os.path.isfile(properties_file):
            raise EnvironmentError("{0} file not found inside {1} directory"
                                   .format(settings.properties_ini,
                                           module_set_path))

        src_os_ver = ModuleSetUtils.get_config_key_value(properties_file,
                                                         src_version_key,
                                                         section_name)
        dst_os_ver = ModuleSetUtils.get_config_key_value(properties_file,
                                                         dst_version_key,
                                                         section_name)
        return [src_os_ver, dst_os_ver]



import unittest
import os

from preupg import settings

dirname = os.path.join(os.path.dirname(__file__), 'tmp')
solution_txt = settings.solution_txt
os.environ['XCCDF_VALUE_TMP_PREUPGRADE'] = dirname
os.environ['CURRENT_DIRECTORY'] = dirname
os.environ['XCCDF_VALUE_REPORT_DIR'] = dirname
os.environ['XCCDF_VALUE_CURRENT_DIRECTORY'] = os.path.join(os.path.dirname(__file__), "..")
os.environ['XCCDF_RESULT_ERROR'] = "3"
os.environ['XCCDF_RESULT_FAILED'] = "2"
os.environ['XCCDF_RESULT_FAIL'] = "2"
os.environ['XCCDF_RESULT_FIXED'] = "1"
os.environ['XCCDF_RESULT_NEEDS_INSPECTION'] = "1"
os.environ['XCCDF_RESULT_NEEDS_ACTION'] = "1"
os.environ['XCCDF_RESULT_NOT_APPLICABLE'] = "10"
os.environ['XCCDF_VALUE_MODULE_PATH'] = "test_script_api"


def suite():
    settings.log_dir = os.getcwd()
    settings.preupg_log = os.path.join(settings.log_dir, "preupg.log")
    settings.preupg_report_log = os.path.join(settings.log_dir, "preupg-report.log")

    suite = unittest.TestSuite()
    from tests import test_preupg
    from tests import test_xml
    from tests import test_generation
    from tests import test_api
    from tests import test_kickstart
    from tests import test_inplace_risks
    from tests import test_creator
    from tests import test_preupg_diff
    suite.addTests(test_preupg.suite())
    suite.addTests(test_xml.suite())
    suite.addTests(test_generation.suite())
    suite.addTests(test_api.suite())
    suite.addTests(test_kickstart.suite())
    suite.addTests(test_inplace_risks.suite())
    suite.addTests(test_creator.suite())
    suite.addTests(test_preupg_diff.suite())
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



# -*- coding: utf-8 -*-

from django import template
register = template.Library()

@register.simple_tag
def url_replace(request, field, value):
    dict_ = request.GET.copy()
    dict_[field] = value
    return dict_.urlencode()



from django import VERSION
from django.conf.urls import patterns, include, url
from django.contrib import admin
from preupg.ui.auth.views import FirstRunView, FirstRunDisableAuthView, login, logout
from preupg.ui.auth.decorators import first_run_required as frr

if VERSION < (1, 7):
    admin.autodiscover()

urlpatterns = patterns('',
    url(r'^', include('preupg.ui.report.urls')),

    #url(r'^settings/', include('preupg.ui.config.urls')),

    url(r"^xmlrpc/", include("preupg.ui.xmlrpc_backend.urls"), name="xmlrpc"),

    # shortcut for XML-RPC submission
    url(r"^submit/$", "preupg.ui.xmlrpc_backend.views.submission_handler", name="xmlrpc-submit"),

    url(r'^admin/', include(admin.site.urls)),

    url(r'^compare/', include('preupg.ui.compare.urls')),

    # private auth URLs
    url(r'^auth/', include('preupg.ui.auth.urls')),

    # public auth URLs -- first display and login itself
    url(r'^login/$', login, {'template_name': 'auth/login.html'}, name='auth-login'),
    url(r'^logout/$', logout, {'template_name': 'auth/logout.html'}, name='auth-logout'),
    url(r'^first/$', frr(FirstRunView.as_view()), name='first-run'),
    url(r'^first/disable/$', frr(FirstRunDisableAuthView.as_view()), name='first-run-disable-auth'),
)



"""
Django settings for preupgrade-assistant UI project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

from django import VERSION

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
DATA_DIR = os.path.join(BASE_DIR, 'data')


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# Make it unique, and don't share it with anybody.
try:
    SECRET_KEY = open(os.path.join(DATA_DIR, 'secret_key')).read()
except:
    from django.utils.crypto import get_random_string
    SECRET_KEY = get_random_string(50, 'abcdefghijklmnopqrstuvwxyz0123456789@#$%^&*(-_=+)')
    open(os.path.join(DATA_DIR, 'secret_key'), 'w').write(SECRET_KEY)


DEBUG   = os.environ.get('DEBUG', False) and True or False
DBDEBUG = os.environ.get('DEBUG', False) == 'DB'
TEMPLATE_DEBUG = os.environ.get('DEBUG', False) == 'TEMPLATE'

ALLOWED_HOSTS = ['*']

# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'preupg.ui.report',
    'preupg.ui.config',
    'preupg.ui.xmlrpc_backend',
    'preupg.ui',
)

if VERSION < (1, 7):
    INSTALLED_APPS += ('south',)
    SOUTH_MIGRATION_MODULES = {
        'preupg.ui.report': 'preupg.ui.report.south_migrations',
        'preupg.ui.config': 'preupg.ui.config.south_migrations',
    }

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)
if VERSION >= (1, 7):
    MIDDLEWARE_CLASSES += (
        'django.contrib.messages.middleware.MessageMiddleware',
    )
MIDDLEWARE_CLASSES += (
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'preupg.ui.exception_middleware.ExceptionMiddleware',
)

ROOT_URLCONF = 'preupg.ui.urls'

WSGI_APPLICATION = 'preupg.ui.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(DATA_DIR, 'db.sqlite'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = 'en-us'

USE_I18N = True

USE_L10N = True

# oscap doesn't provide TZ aware dates anyway
USE_TZ = False

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(DATA_DIR, 'static')

MEDIA_URL = '/upload/'
MEDIA_ROOT = os.path.join(DATA_DIR, 'upload')

RESULTS_DIR = os.path.join(DATA_DIR, 'results')


from django.conf.global_settings import TEMPLATE_CONTEXT_PROCESSORS
TEMPLATE_CONTEXT_PROCESSORS += (
    'django.core.context_processors.request',
    'preupg.ui.auth.context_processors.auth_enabled',
)


AUTHENTICATION_BACKENDS = (
    'preupg.ui.auth.backends.AutologinBackend',
    'django.contrib.auth.backends.ModelBackend',
)

LOGIN_URL = 'auth-login'
LOGIN_REDIRECT_URL = '/'
LOGIN_EXEMPT_URLS = ['^xmlrpc/', '^submit/', '^login/', '^admin/', '^first/']

if DEBUG:
    opt_apps = ['django_extensions', 'debug_toolbar']
    for mod in opt_apps:
        try:
            __import__(mod)
        except ImportError:
            pass
        else:
            INSTALLED_APPS += (mod,)


XMLRPC_METHODS = {
    'submission': (
        ('preupg.ui.xmlrpc.submission', 'submit'),
    ),
}


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': DEBUG and 'DEBUG' or 'INFO',
            'propagate': True,
        },
        'django.db.backends': {
            'level': DBDEBUG and 'DEBUG' or 'INFO',
            'propagate': True,
        },
    },
}




# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.urlresolvers import reverse, reverse_lazy
from django.http.response import HttpResponseRedirect, Http404
from django.utils.http import is_safe_url

from django.views.generic.list import ListView

from .forms import UserCreationForm
from preupg.ui.config.models import AppSettings
from preupg.ui.utils.views import return_error

from django.views.generic import TemplateView, DeleteView, FormView, View
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login as auth_login, get_user_model
from django.contrib.auth.views import login as auth_login_view, logout
from django.contrib.auth.models import User
from django.shortcuts import redirect, resolve_url, get_object_or_404
from django.forms.forms import NON_FIELD_ERRORS
from django.forms.util import ErrorList


def login(request, **kwargs):
    if get_user_model().objects.count() == 0:
        return redirect('first-run')
    else:
        # tries to authenticate without credentials (with AutologinBackend)
        u = authenticate()
        if u is not None:
            auth_login(request, u)
            redirect_to = request.REQUEST.get(kwargs.get('redirect_field_name', REDIRECT_FIELD_NAME), '')
            if not is_safe_url(url=redirect_to, host=request.get_host()):
                redirect_to = resolve_url(settings.LOGIN_REDIRECT_URL)
            return HttpResponseRedirect(redirect_to)
        else:
            return auth_login_view(request, **kwargs)


class AuthListView(ListView):
    template_name = "auth/user_list.html"
    queryset = User.objects.all()

    def get_context_data(self, **kwargs):
        context = super(AuthListView, self).get_context_data(**kwargs)
        context['title'] = 'User List'
        return context


class GenericCreateUserView(FormView, TemplateView):
    """View for creating users"""
    # this View should not be rendered directly

    def post(self, request, *args, **kwargs):
        context = {}
        context['title'] = 'Create User'
        context['form'] = UserCreationForm(request.POST)
        if context['form'].is_valid():
            if User.objects.filter(username=context['form'].cleaned_data['username']).exists():
                errors = context['form']._errors.setdefault(NON_FIELD_ERRORS, ErrorList())
                errors.append(u"There is already a user with such username.")
                return self.render_to_response(context)
            User.objects.create_superuser(
                context['form'].cleaned_data['username'],
                context['form'].cleaned_data['email'],
                context['form'].cleaned_data['password']
            )

            try:
                return HttpResponseRedirect(request.GET['next'])
            except KeyError:
                pass
            return redirect('auth-list')
        return self.render_to_response(context)

    def get(self, request, *args, **kwargs):
        context = {}
        context['title'] = 'Create User'
        context.update(self.get_context_data(**kwargs))
        context['form'] = UserCreationForm()
        return self.render_to_response(context)


class FirstRunView(GenericCreateUserView):
    template_name = "auth/first_run.html"

    def get_context_data(self, **kwargs):
        context = super(FirstRunView, self).get_context_data(**kwargs)
        context['title'] = 'Welcome to Preupgrade Assistant UI'
        return context

    def post(self, request, *args, **kwargs):
        AppSettings.unset_autologin_user_id()
        return super(FirstRunView, self).post(request, *args, **kwargs)


class CreateUserView(GenericCreateUserView):
    template_name = "auth/create_user.html"


class EditUserView(GenericCreateUserView):
    """View for editing users"""
    template_name = "auth/create_user.html"

    def get_context_data(self, **kwargs):
        context = super(EditUserView, self).get_context_data(**kwargs)
        context['edit'] = True
        try:
            context['title'] = "Edit User '%s'" % kwargs['username']
        except KeyError:
            raise Http404("There is no such user.")
        context['post_url_name'] = reverse("auth-edit", args=(kwargs['username'], ))
        return context

    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        context['form'] = UserCreationForm(request.POST)
        if context['form'].is_valid():
            user = get_object_or_404(User, username=context['form'].cleaned_data['username'])
            user.password = context['form'].cleaned_data['password']
            user.email = context['form'].cleaned_data['email']
            user.save()
            return redirect('auth-list')
        return self.render_to_response(context)

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        user = get_object_or_404(User, username=kwargs['username'])
        context['form'] = UserCreationForm(
            {'username': user.username, 'email': user.email},
            edit_mode=True,
        )
        context['next'] = 'index'
        return self.render_to_response(context)


class GenericAuthMngmntView(View):
    def get_redirect(self):
        try:
            return HttpResponseRedirect(self.request.GET['next'])
        except KeyError:
            pass
        return redirect('results-list')


class FirstRunDisableAuthView(GenericAuthMngmntView):
    def get(self, request, *args, **kwargs):
        user = get_user_model()(username='autologin')
        user.save()
        AppSettings.set_autologin_user_id(user.id)
        return self.get_redirect()


class EnableAuthView(GenericAuthMngmntView):
    def get(self, request, *args, **kwargs):
        AppSettings.unset_autologin_user_id()
        return self.get_redirect()


class DisableAuthView(GenericAuthMngmntView):
    def get(self, request, *args, **kwargs):
        AppSettings.set_autologin_user_id(request.user.id)
        return self.get_redirect()


class DeleteAuthView(DeleteView):
    model = User
    slug_field = 'username'
    slug_url_kwarg = 'username'
    success_url = reverse_lazy('auth-list')




"""
The class is used for common stuff issues like
generating common logs, coping these common logs
to assessment
"""

from __future__ import unicode_literals
import os
import datetime
import shutil
from distutils import dir_util
from preupg.utils import FileHelper, DirHelper, ProcessHelper
from preupg.utils import SystemIdentification
from preupg.logger import log_message
from preupg import settings


class Common(object):

    """Class handles with common log files"""

    def __init__(self, conf):
        self.conf = conf
        self.cwd = ""
        self.lines = FileHelper.get_file_content(self.conf.common_scripts,
                                                 "rb", True)
        self.common_result_dir = ""

    def common_logfiles(self, filename):
        """build path for provided filename"""
        return os.path.join(self.get_common_dir(), filename)

    def get_common_dir(self):
        """Function returns common dir"""
        return os.path.join(self.conf.cache_dir, settings.common_name)

    def switch_dir(self):
        """Switch to current directory"""
        com_dir = self.get_common_dir()
        if not os.path.exists(com_dir):
            DirHelper.check_or_create_temp_dir(com_dir)
        self.cwd = os.getcwd()
        os.chdir(self.get_common_dir())

    def switch_back_dir(self):
        """Function switch back to self.cwd"""
        os.chdir(self.cwd)

    def common_results(self):
        """run common scripts"""
        log_message("Gathering logs used by the Preupgrade Assistant:")
        self.switch_dir()
        try:
            max_length = max(max([len(x.split("=", 4)[3]) for x in self.lines]), len(settings.assessment_text))
            # Log files which will not be updated
            # when RPM database is not changed
            for counter, line in enumerate(self.lines):
                line = line.strip()
                if line.startswith("#"):
                    continue
                cmd, log_file, dummy_bash_value, name, values = line.split("=", 4)
                log_message("%s : %.2d/%d ...running" % (name.ljust(max_length),
                                                         counter+1,
                                                         len(self.lines)),
                            new_line=False)
                start_time = datetime.datetime.now()
                common_file_path = self.common_logfiles(log_file)
                ProcessHelper.run_subprocess(cmd, output=common_file_path, shell=True)
                end_time = datetime.datetime.now()
                diff = end_time - start_time
                log_message(" %sfinished (time %.2d:%.2ds)" % ('\b' * 8,
                                                               diff.seconds / 60,
                                                               diff.seconds % 60))
                # os.chmod(common_file_path, 0640)
            self.switch_back_dir()
        except IOError:
            return 0
        else:
            return 1

    def copy_common_files(self):
        """run common scripts"""
        self.switch_dir()

        try:
            for line in self.lines:
                if line.strip().startswith("#"):
                    continue
                values = line.strip().split("=", 5)
                if values[4] == "YES":
                    shutil.copyfile(values[1],
                                    os.path.join(self.conf.assessment_results_dir,
                                                 "kickstart",
                                                 values[5]))
                else:
                    if os.path.exists(os.path.join(self.conf.assessment_results_dir,
                                                   values[1])):
                        os.remove(values[1])
        except IOError:
            return 0
        else:
            self.switch_back_dir()
            return 1



# -*- coding: utf-8 -*-

import logging
import os
import datetime
import tarfile
import shutil

from .models import Test, TestResult, HostRun, Result, Address, TestLog, TestGroup, TestGroupResult
from .models import Risk

from processing import parse_xml_report, update_html_report

from django.db import transaction
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse


DATE_FORMAT = "%Y-%m-%dT%H:%M:%S"

logger = logging.getLogger('preup_ui')


def filter_files_by_ext(tar_content, ext, error_msg):
    try:
        return filter(lambda x: x.name.endswith(ext) and x.name.count('/') <= 1, tar_content)[0]
    except IndexError:
        raise Exception(error_msg)


def remove_upload(path):
    """ add exc handling & logging """
    abs_path = os.path.abspath(path)
    if not abs_path.startswith(settings.MEDIA_ROOT):
        logger.error("Upload is not in MEDIA_ROOT, canceling cleanup process")
        return
    tb_dir = os.path.dirname(abs_path)
    if not os.path.samefile(settings.MEDIA_ROOT, tb_dir):
        shutil.rmtree(tb_dir)
    else:
        os.unlink(abs_path)


def extract_tarball(tbpath, target_dir):
    """ return report paths: (xml, html) """
    def strip_subfolder(tar_content):
        """The Web UI does not expect the subfolder in the extracted tarball
        content.
        """
        for member in tar_content:
            path_without_subfolder = member.path.split(os.sep)[1:]
            if path_without_subfolder:
                member.path = os.path.join(*member.path.split(os.sep)[1:])
                yield member

    tar = tarfile.open(tbpath)
    tar_content = tar.getmembers()
    tar.extractall(target_dir, strip_subfolder(tar_content))
    tar.close()

    xml = filter_files_by_ext(tar_content, '.xml',
                              'Missing XML report in tarball.')
    html = filter_files_by_ext(tar_content, '.html',
                               'Missing HTML report in tarball.')

    xml_path = os.path.join(target_dir, xml.name)
    html_path = os.path.join(target_dir, html.name)
    return xml_path, html_path


class ReportImporter(object):
    """
    Imports report on provided path to database
    """

    def __init__(self, tb_path, hostrun_id):
        """ create all required global DB models """
        self.tb_path = tb_path
        hostrun = get_object_or_404(HostRun, id=hostrun_id)
        self.hostrun = hostrun
        self.run = hostrun.run
        self.result = self._create_result()

        # initialized in execute_import -> process_tarball
        self.parsed_data = None
        # initialized in process_tarball
        self.html_path = None

    def _create_result(self):
        result = Result()
        result.hostrun = self.hostrun
        result.save()
        return result

    def _update_result(self):
        """ update result with data from report """
        def set_date(obj, field, key):
            try:
                setattr(obj, field, datetime.datetime.strptime(self.parsed_data[key], DATE_FORMAT))
            except KeyError:
                pass

        self.result.filename = os.path.basename(self.html_path)
        set_date(self.result, 'dt_submitted', 'started')
        set_date(self.result, 'dt_finished', 'finished')
        self.result.identity = self.parsed_data['identity']
        self.result.hostname = self.parsed_data['host']
        self.result.save()

    def _process_tarball(self):
        xml_path, html_path = extract_tarball(self.tb_path, self.result.get_result_dir())
        self.html_path = html_path
        remove_upload(self.tb_path)
        update_html_report(self.html_path)
        return parse_xml_report(xml_path)

    @transaction.commit_on_success
    def _add_to_db(self):
        """ add data to dabase """
        for address in self.parsed_data['addresses']:
            Address(address=address, result=self.result).save()

        test_keys = ['id_ref', 'title', 'description', 'fix', 'fix_type', 'fixtext']
        group_keys = ['xccdf_id', 'title', ]
        # cache of processed groups -- efficient way of assigning parents
        groups = {}
        for group in self.parsed_data['groups']:
            group_dict = dict((key, group[key]) for key in group_keys if key in group)
            if 'parent' in group:
                group_dict['parent'] = groups[group['parent']][0]
            tg = TestGroup.objects.create(**group_dict)

            trg = TestGroupResult()
            trg.group = tg
            trg.result = self.result
            if 'parent' in group:
                trg.parent = groups[group['parent']][1]
            trg.save()
            trg.root = trg.get_root()

            # save group and group result
            groups[group_dict['xccdf_id']] = (tg, trg)

            for rule in group['rules']:
                test_dict = dict((key, rule[key]) for key in test_keys if key in rule)
                test_dict['group'] = tg
                try:
                    # links in fixtext miss id of result, see report/processing.py stringify_children
                    test_dict['fixtext'] = test_dict['fixtext'].replace(
                        '__INSERT_URL__',
                        reverse('show-file', args=(self.result.id,))
                    )
                except KeyError:
                    pass
                t = Test.objects.create(**test_dict)

                tr = TestResult()
                try:
                    tr.set_state(rule['result'], False)
                except KeyError:
                    # rule wasn't selected probably
                    continue
                tr.date = datetime.datetime.strptime(rule['time'], DATE_FORMAT)
                tr.test = t
                tr.group = trg
                tr.result = self.result
                tr.root_group = trg.get_root()
                tr.save()

                # add logs to DB
                if 'logs' in rule:
                    TestLog.objects.bulk_create_logs(rule['logs'], tr)
                if 'risks' in rule:
                    Risk.objects.bulk_create_logs(rule['risks'], tr)

    def _calculate_group_stats(self):
        groups = []
        def recurs(parent, child):
            for gch in child.children():
                recurs(child, gch)
            child.failed_test_count += TestResult.objects.for_tgr(child).failed().count()
            child.ni_test_count += TestResult.objects.for_tgr(child).ni().count()
            child.na_test_count += TestResult.objects.for_tgr(child).na().count()
            child.test_count += TestResult.objects.for_tgr(child).count()
            if parent:
                parent.failed_test_count += child.failed_test_count
                parent.ni_test_count += child.ni_test_count
                parent.na_test_count += child.na_test_count
                parent.test_count += child.test_count
            groups.append(child)

        for group in self.result.groups:
            if group in groups:
                continue
            recurs(None, group)

        for group in groups:
            group.save()

    def _calculate_result_stats(self):
        self.result.test_count = TestResult.objects.for_result(self.result).count()
        self.result.failed_test_count = TestResult.objects.for_result(self.result).failed().count()
        self.result.na_test_count = TestResult.objects.for_result(self.result).na().count()
        self.result.ni_test_count = TestResult.objects.for_result(self.result).ni().count()
        self.result.save()

    def _calculate_stats(self):
        """
        calculate helpful stats functions, like sums
         -- this is best to be done, when everything's in DB
        """
        self._calculate_group_stats()
        self._calculate_result_stats()

    def execute_import(self):
        """ execute import itself, this is the main call """
        self.parsed_data = self._process_tarball()
        self._update_result()
        self._add_to_db()

        self.hostrun.set_finished()
        self.hostrun.set_risk()
        if self.run.all_done():
            self.run.finish()

        self._calculate_stats()


def import_report(tb_path, hostrun_id):
    ri = ReportImporter(tb_path, hostrun_id)
    ri.execute_import()


def main():
    pass


if __name__ == '__main__':
    main()



from __future__ import print_function, unicode_literals

import os
import sys

from preupg.xml_manager import html_escape
from preupg.utils import FileHelper, ModuleSetUtils
from preupg import settings
from preupg.xmlgen import xml_tags
from preupg.xmlgen.script_utils import ModuleHelper
from preupg.exception import EmptyTagGroupXMLError


def module_path_from_root_dir(dirname, module_set_dir):
    """
    Remove module_set_dir from dirname path

    @param {str} dir_name - directory of specific module or module-set
        directory
    @param {str} module_set_dir - directory where all modules are stored

    @return {list} - splited path to module

    @example
    >>> module_path_from_root_dir(/root/RHEL6_7-results/selinux/CustomPolicy,
                                  /root/RHEL6_7-results)
    ['selinux', 'CustomPolicy']
    """
    return (dirname
            .replace(module_set_dir, '', 1)
            .lstrip(os.path.sep)  # remove first / from path
            .split(os.path.sep))


class XmlUtils(object):
    """Class generate a XML from xml_tags and loaded INI file"""
    def __init__(self, module_set_dir, module_dir, ini_files):
        """
        @param {str} module_set_dir - directory where all modules are stored
        @param {str} module_dir - directory of specific module or module-set
            directory
        @param {dict} ini_files - ini file options and their values in format:
            {ini_file_path: {option1: value, option2: value, ...}}
        """
        self.module_set_dir = module_set_dir
        self.module_dir = module_dir
        self.ini_files = ini_files
        self.select_rules = []
        self.rule = []
        self._test_config_file()
        self.mh = None

    def _test_config_file(self):
        """
        Checks if there are any illegal tags in config files i.e.
        (group.ini, module.ini) and print out warning message about them
        """
        allowed_tags = set(['content_description', 'content_title',
                            'applies_to', 'author', 'binary_req', 'bugzilla',
                            'config_file', 'group_title', 'mode', 'requires'])
        for ini_file, ini_content in iter(self.ini_files.items()):
            ini_content_tags = set(ini_content.keys())
            different = ini_content_tags.difference(allowed_tags)
            if different:
                sys.stderr.write("Warning: tags: '{0}' are not allowed "
                                 "in config file {1}.\n"
                                 "Allowed tags are {2}.\n"
                                 .format(', '
                                         .join(str(tag) for tag in different),
                                         ini_file,
                                         ', '.join(allowed_tags)))

    def update_files(self, file_name, content):
        """Function updates file_name <migrate or update> according to INI
        file.

        :param file_name: specified in INI file like mode: upgrade, migrate
        :param content: name of the content like xccdf_rule_...
        :return: Nothing
        """
        path_name = os.path.join(settings.UPGRADE_PATH, file_name)
        lines = []
        if os.path.exists(path_name):
            lines = FileHelper.get_file_content(path_name, 'rb', method=True)
        test_content = [x.strip() for x in lines if content in x.strip()]
        if not test_content:
            lines.append(content + '\n')
            FileHelper.write_to_file(path_name, 'wb', lines)

    def _update_check_description(self, filename):
        new_text = []
        lines = FileHelper.get_file_content(os.path.join(self.module_dir,
                                                         filename), "rb", True)

        bold = '<xhtml:b>{0}</xhtml:b>'
        br = '<xhtml:br/>'
        table_begin = '<xhtml:table>'
        table_end = '</xhtml:table>'
        table_header = '<xhtml:tr><xhtml:th>Result</xhtml:th><xhtml:th>' \
                       'Description</xhtml:th></xhtml:tr>'
        table_row = '<xhtml:tr><xhtml:td>{0}</xhtml:td><xhtml:td>{1}' \
                    '</xhtml:td></xhtml:tr>'
        new_text.append(br + br + '\n' + bold.format('Details:') + br)
        results = False
        for line in lines:
            if '=' in line:
                if not results:
                    new_text.append(bold.format('Expected results:') + br)
                    new_text.append(table_begin + '\n' + table_header)
                    results = True
                try:
                    exp_results = line.strip().split('=')
                    new_text.append(table_row.format(exp_results[0],
                                                     exp_results[1]) + '\n')
                except IndexError:
                    pass
            else:
                new_text.append(line.rstrip() + br)
        if results:
            new_text.append(table_end + '\n')

        return '\n'.join(new_text)

    def update_values_list(self, section, search_exp, replace_exp):
        """
        The function replaces tags taken from INI files.
        Tags are mentioned in xml_tags.py
        """
        forbidden_empty = ["{scap_name}", "{main_dir}"]
        if search_exp == "{content_description}":
            replace_exp = replace_exp.rstrip()
        elif search_exp == "{check_description}":
            replace_exp = '\n' + replace_exp + '\n'
        elif search_exp == "{config_file}":
            new_text = ""
            for lines in replace_exp.split(','):
                new_text += "<xhtml:li>" + lines.strip() + "</xhtml:li>"
            replace_exp = new_text.rstrip()
        elif search_exp == "{solution_text}":
            new_text = "_" + '_'.join(
                module_path_from_root_dir(self.module_dir,
                                          self.module_set_dir))\
                       + "_SOLUTION_MSG"
            replace_exp = new_text
        if replace_exp == '' and search_exp in forbidden_empty:
            raise EmptyTagGroupXMLError(search_exp)

        for cnt, line in enumerate(section):
            if search_exp in line:
                section[cnt] = line.replace(search_exp, replace_exp)

    def add_value_tag(self):
        """The function adds VALUE tag in group.xml file"""
        value_tag = []
        check_export_tag = []

        check_export_tag.append(xml_tags.RULE_SECTION_VALUE_IMPORT)
        for key, val in xml_tags.DIC_VALUES.items():
            value_tag.append(xml_tags.VALUE)
            if key == 'current_directory':
                val = '/'.join(module_path_from_root_dir(self.module_dir,
                                                         self.module_set_dir))
                val = 'SCENARIO/' + val
            if key == 'module_path':
                val = '_'.join(module_path_from_root_dir(self.module_dir,
                                                         self.module_set_dir))
            self.update_values_list(value_tag, "{value_name}", val)
            self.update_values_list(value_tag, "{val}", key.lower())
            check_export_tag.append(xml_tags.RULE_SECTION_VALUE)
            self.update_values_list(check_export_tag, "{value_name_upper}",
                                    key.upper())
            self.update_values_list(check_export_tag, "{val}", key.lower())
        for key, val in xml_tags.GLOBAL_DIC_VALUES.items():
            check_export_tag.append(xml_tags.RULE_SECTION_VALUE_GLOBAL)
            self.update_values_list(check_export_tag, "{value_name_upper}",
                                    key.upper())
            self.update_values_list(check_export_tag, "{value_name}", key)

        return value_tag, check_export_tag

    def check_script_modification(self, key, k):
        """Function checks a check script"""
        self.mh.check_scripts(k)
        check_func = {'log_': ['log_slight_risk',
                               'log_medium_risk', 'log_high_risk',
                               'log_extreme_risk', 'log_info',
                               'log_error', 'log_warning'],
                      'exit_': ['exit_error', 'exit_fail',
                                'exit_fixed', 'exit_not_applicable',
                                'exit_pass', 'exit_informational']
                      }
        for check in check_func:
            self.mh.check_inplace_risk(prefix=check,
                                       check_func=check_func[check])
        self.update_values_list(self.rule, "{scap_name}",
                                settings.check_script)
        requirements = {'applies_to': 'check_applies',
                        'binary_req': 'check_bin',
                        'requires': 'check_rpm'}
        updates = dict()
        for req in requirements:
            if req in key:
                updates[requirements[req]] = key[req]
        if 'author' in key:
            author = key['author']
        else:
            author = None
        self.mh.update_check_script(updates, author=author)
        self.update_values_list(self.rule, "{" + k + "}",
                                settings.check_script)

    def prepare_sections(self):
        """
        The function prepares all tags needed for generation group.xml file.
        """
        for ini_filepath, ini_file_content in iter(self.ini_files.items()):
            if ini_filepath.endswith("group.ini"):
                self.rule.append(xml_tags.GROUP_INI)
                self.update_values_list(self.rule, '{group_title}',
                                        ini_file_content['group_title'])
                self.update_values_list(self.rule, '{group_value}', "")
            else:
                self.rule.append(xml_tags.CONTENT_INI)
                self.create_xml_from_ini(ini_filepath, ini_file_content)
                self.update_values_list(self.rule, "{select_rules}",
                                        ' '.join(self.select_rules))
            self.update_values_list(self.rule, "{main_dir}",
                                    '_'.join(module_path_from_root_dir(
                                        self.module_dir,
                                        self.module_set_dir)))
        return self.rule

    def fnc_config_file(self, key, name):
        """Function updates a config file."""
        if name in key and key[name] is not None:
            self.update_values_list(self.rule, "{config_section}",
                                    xml_tags.CONFIG_SECTION)
            self.update_values_list(self.rule, "{config_file}",
                                    key['config_file'])
        else:
            self.update_values_list(self.rule, "{config_section}", "")

    def fnc_check_script(self, key, name):
        """
        Function updates a check_script

        @param {dict} key - options and values from ini file:
            {option1: value, option2: value, ...}
        @param {str} name - 'check_script'
        """
        self.check_script_modification(key, name)
        self.update_values_list(self.select_rules, "{scap_name}",
                                settings.check_script)

    def fnc_check_description(self, key, name):
        """ Function updates a check_description """
        if name in key and key[name] is not None:
            escaped_text = self._update_check_description(key[name])
            self.update_values_list(self.rule, "{check_description}",
                                    escaped_text)
        else:
            self.update_values_list(self.rule, "{check_description}", "")

    def fnc_solution_text(self, *_):
        """
        Function updates a solution text.

        @param {dict} key - options and values from ini file:
            {option1: value, option2: value, ...}
        @param {str} name - 'solution'
        """
        self.mh.check_scripts('solution')
        self.update_values_list(self.rule, "{solution_text}", "")

    def fnc_update_mode(self, name):
        """
        Function update <upgrade_path>/<migrate.conf|update.conf> files
        migrate_xccdf_path
        :param key:
        :param name:
        :return:
        """

        content = "{rule}{main_dir}_{name}".format(
            rule=xml_tags.TAG_RULE,
            main_dir='_'.join(module_path_from_root_dir(self.module_dir,
                                                        self.module_set_dir)),
            name=settings.check_script)
        if not name:
            self.update_files('migrate', content)
            self.update_files('upgrade', content)
        for x in name.split(','):
            self.update_files(x.strip(), content)

    def dummy_fnc(self, key, name):
        """Function is only dummy."""
        pass

    def update_text(self, key, name):
        """Function updates a text."""
        if key[name] is not None:
            # escape values so they can be loaded as XMLs
            escaped_text = html_escape(key[name])
            self.update_values_list(self.rule, "{" + name + "}", escaped_text)

    def create_xml_from_ini(self, ini_filepath, ini_content):
        """
        Creates group.xml file from INI file. All tags are replaced by function
        update_value_list. Function also checks whether check script fulfills
        all criteria

        @param {str} ini_filepath - real path of ini file
        @param {dict} ini_content - options and values from ini file:
            {option1: value, option2: value, ...}
        @throws {IOError}
        """
        self.select_rules.append(xml_tags.SELECT_TAG)
        update_fnc = {
            'config_file': self.fnc_config_file,
            'check_script': self.fnc_check_script,
            'check_description': self.fnc_check_description,
            'solution': self.fnc_solution_text,
            'content_title': self.update_text,
            'content_description': self.update_text,
        }
        ModuleHelper.check_required_fields(ini_filepath, ini_content)
        self.mh = ModuleHelper(os.path.dirname(ini_filepath))

        self.update_values_list(self.rule, "{rule_tag}",
                                ''.join(xml_tags.RULE_SECTION))
        value_tag, check_export_tag = self.add_value_tag()
        self.update_values_list(self.rule, "{check_export}",
                                ''.join(check_export_tag))
        self.update_values_list(self.rule, "{group_value}",
                                ''.join(value_tag))

        for k, function in iter(update_fnc.items()):
            try:
                function(ini_content, k)
            except IOError as e:
                err_msg = "Invalid value of the field '%s' in INI file" \
                          " '%s'\n'%s': %s\n" % (k, ini_filepath,
                                                 ini_content[k], e.strerror)
                raise IOError(err_msg)

        self.update_values_list(
            self.rule, '{group_title}',
            html_escape(ini_content['content_title'])
        )
        if 'mode' not in ini_content:
            self.fnc_update_mode('migrate, upgrade')
        else:
            self.fnc_update_mode(ini_content['mode'])



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse

import os
import logging

from utils.scan import run_subprocess
from utils.distrib import run_remotely


logger = logging.getLogger('preup_ui')


class DummyConf(object):
    pass


def get_local_conf():
    c = DummyConf()
    c.skip_common = False
    c.quiet = False
    #c.profile = "xccdf_org.preupgrade-content_profile_fedora-default"
    c.list = None
    c.scan = 'RHEL6_7'
    c.force = True
    return c


def build_command():
    """ Build command which triggers analysis """


def run_locally(request):
    """
    DEPRECATED: do not use!
    suid binary is not being compiled atm because it's insecure

    Run analysis locally
    """
    #command = ["preupg_runner", "--force", "--upload"]
    command.append(
        request.build_absolute_uri(reverse('xmlrpc-submit'))
    )
    logger.info("Starting scan.")
    rvalue = run_subprocess(command, log=False)
    if rvalue != 0:
        logger.error("Scan failed.")
        raise RuntimeError("""\
Scan wasn't successful. If you have SELinux enabled, \
please make sure that SELinux boolean 'httpd_run_preupgrade' is enabled.""")


def run(request):
    """
    Entry point. Run scan distributively on provided set of hosts
    """
    #if run_object.has_local():
    run_locally(request)
    #for remote in run_object.remotes():
    #    print 'running on remote', remote
    #    run_remotely(remote.host.hostname, remote.host.ssh_name, remote.host.ssh_password, remote.id)


def main():
    pass

if __name__ == '__main__':
    main()



from django.contrib.auth import get_user_model
from django.db import models

import json

class AppSettings(models.Model):
    """
    settings stored in DB

    for bools, value is either 'T' or 'F'
    """
    key = models.CharField(max_length=32, primary_key=True)
    value = models.TextField(blank=True, null=True)

    def __unicode__(self):
        return "%s = '%s'" % (self.key, self.value)

    @classmethod
    def get_initial_state_filter(cls):
        """tests with these states should be displayed initially"""
        # TODO: when content is invalid, reset it to default _valid_ value
        try:
            value = cls.objects.get(key="STATES_FILTER").value
        except Exception:
            pass
        else:
            if value:
                try:
                    return json.loads(value)
                except ValueError:
                    pass
        s = cls.set_initial_state_filter(["error", "fail", "needs_action", "needs_inspection", "fixed", "pass", "informational"])
        return json.loads(s.value)

    @classmethod
    def set_initial_state_filter(cls, value):
        """
        tests with these states should be displayed initially
        value: list of states [fail, error, ...]
        """
        try:
            json_value = json.dumps(value)
        except TypeError:
            json_value = json.dumps(list(value))
        # remove all previous filters
        cls.objects.filter(key="STATES_FILTER").delete()
        #set new one
        s = cls(key="STATES_FILTER", value=json_value)
        s.save()
        return s

    @classmethod
    def get_autologin_user_id(cls):
        """ returns user to be auto-logged in """
        try:
            return int(cls.objects.get(key="AUTOLOGIN_USER").value)
        except cls.DoesNotExist:
            return None

    @classmethod
    def set_autologin_user_id(cls, user_id):
        """ sets user to be auto-logged in """
        cls(key="AUTOLOGIN_USER", value=str(user_id)).save()

    @classmethod
    def unset_autologin_user_id(cls):
        """ unsets user to be auto-logged in """
        cls.objects.filter(key="AUTOLOGIN_USER").delete()




# -*- coding: utf-8 -*-
"""
The application module serves for creating a content
"""

from preupg.logger import logging, LoggerHelper, logger
from preupg.creator.ui_helper import UIHelper


class Application(object):

    """Class for oscap binary and reporting results to UI"""

    def __init__(self, conf):
        """conf is preupg.conf.Conf object, contains configuration"""
        self.conf = conf
        if self.conf.debug is None:
            LoggerHelper.add_stream_handler(logger, logging.INFO)
        else:
            LoggerHelper.add_stream_handler(logger, logging.DEBUG)
        self.ui_helper = UIHelper(self.conf.maindir)

    def run(self):
        self.ui_helper.take_manadatory_info()



from __future__ import print_function, unicode_literals

import os
import re

from preupg.utils import FileHelper
from preupg import settings
from preupg.exception import MissingHeaderCheckScriptError
from preupg.exception import MissingFileInContentError, MissingTagsIniFileError


class ModuleHelper(object):

    def __init__(self, dir_name):
        self.dir_name = dir_name
        self.check_script_path = os.path.join(self.dir_name,
                                              settings.check_script)
        self.solution_txt_path = os.path.join(self.dir_name,
                                              settings.solution_txt)

    def get_check_script_path(self):
        return self.check_script_path

    def get_solution_txt_path(self):
        return self.solution_txt_path

    def check_scripts(self, type_name):
        """
        The function checks whether script exists in content directory
        If check_script exists then the script checks whether it is executable

        @param {str} type_name - name of the script e.g. solution
            or check_script
        @throws {MissingFileInContentError}
        """
        if type_name == 'check_script':
            if not os.path.isfile(self.check_script_path):
                raise MissingFileInContentError(file=self.check_script_path,
                                                dir=self.dir_name)
            FileHelper.check_executable(self.check_script_path)
        else:
            if not os.path.isfile(self.solution_txt_path):
                raise MissingFileInContentError(file=self.solution_txt_path,
                                                dir=self.dir_name)

    @staticmethod
    def apply_function(updates, begin_fnc, end_fnc, dummy_sep, script_type):
        """
        The function generates check_applies_to function into check_scripts
        mentioned in BASH or Python
        """
        template = 'check_applies_to ' + begin_fnc
        if script_type == "sh":
            template += '"' + updates['check_applies'] + '"'
        else:
            template += 'check_applies="' + updates['check_applies'] + '"'
        template += end_fnc
        return template

    @staticmethod
    def rpm_bin_function(updates, begin_fnc, end_fnc, sep, script_type):
        """
        The function generates check_rpm_to function into check_scripts
        mentioned in BASH or Python
        """
        if script_type == "python":
            fnc_name = ['check_rpm="', 'check_bin="']
        else:
            fnc_name = ['"', '"']
        template = 'check_rpm_to ' + begin_fnc + fnc_name[0]

        if "check_rpm" in updates and updates['check_rpm'] is not None:
            template += updates['check_rpm'] + '"'
        else:
            template += '"'
        template += sep + fnc_name[1]
        if "check_bin" in updates and updates['check_bin'] is not None:
            template += updates['check_bin'] + '"'
        else:
            template += '"'
        template += end_fnc

        return template

    @staticmethod
    def generate_common_stuff(lic, updates, script_type):
        """
        Function generates a common stuff
        for Python and BASH scripts
        """
        generated_section = []
        functions = ['check_applies_to', 'check_rpm_to', 'switch_to_content']
        pre_comment = ""
        begin_fnc = ""
        end_fnc = ""
        sep = " "
        if script_type == "sh":
            pre_comment = '#'
        else:
            lic = '"""' + lic + '"""'
            begin_fnc = "("
            end_fnc = ")"
            sep = ","
        if settings.autocomplete:
            for row_lic in lic.split('\n'):
                functions.append(pre_comment + row_lic)
                generated_section.append(pre_comment + row_lic)
        if script_type == "sh":
            functions.append('/usr/share/preupgrade/common.sh')
            generated_section.append('. /usr/share/preupgrade/common.sh')
        if "check_applies" in updates:
            generated_section.append(
                ModuleHelper.apply_function(updates, begin_fnc,
                                            end_fnc, sep, script_type)
            )
        if "check_bin" in updates or "check_rpm" in updates:
            generated_section.append(
                ModuleHelper.rpm_bin_function(updates, begin_fnc,
                                              end_fnc, sep, script_type)
            )
        return generated_section, functions

    def update_check_script(self, updates, author=None):
        """
        The function updates check script with license file
        and with API functions like check_rpm_to and check_applies_to
        """
        script_type = FileHelper.get_script_type(self.check_script_path)
        if author is None:
            author = "<empty_line>"
        generated_section, functions = ModuleHelper.generate_common_stuff(
            settings.license % author, updates, script_type)
        lines = FileHelper.get_file_content(self.check_script_path, "rb",
                                            method=True)
        if not [x for x in lines if re.search(r'#END GENERATED SECTION', x)]:
            raise MissingHeaderCheckScriptError(self.check_script_path)
        for func in functions:
            lines = [x for x in lines if func not in x.strip()]
        output_text = ""
        for line in lines:
            if '#END GENERATED SECTION' in line:
                new_line = '\n'.join(generated_section)
                new_line = new_line.replace('<empty_line>',
                                            '').replace('<new_line>', '')
                output_text += new_line + '\n'
            output_text += line
        FileHelper.write_to_file(self.check_script_path, "wb", output_text)

    def check_inplace_risk(self, prefix="", check_func=None):
        """
        The function checks inplace risks
        in check_script and informs user in case of wrong usage
        """
        if check_func is None:
            check_func = []
        lines = FileHelper.get_file_content(self.check_script_path, "rb")
        compile_req = re.compile(r'^#', re.M | re.I)
        lines = [x for x in lines if not compile_req.search(x.strip())]
        inplace_lines = [x for x in lines if prefix in x]
        for line in inplace_lines:
            tags = [x for x in check_func
                    if re.findall('\\b' + x.split()[0] + '\\b', line)]
            if not tags:
                continue
            checks = [x for x in check_func if len(x.split()) > 1]
            tags = [x for x in checks
                    if re.findall('\\b' + x.split()[1] + '\\b', line)]
            if not tags:
                continue

    @staticmethod
    def check_required_fields(ini_filepath, fields_in_ini):
        """
        The function checks whether all the required fields are used in an INI
        file.

        @param {str} ini_filepath - real path of ini file
        @param {dict} fields_in_ini - options and values from ini file:
            {option1: value, option2: value, ...}
        @throws {MissingTagsIniFileError} - when required option is missing
            in ini file
        """
        required_fields = ['content_title', 'content_description']
        not_in_ini = [x for x in required_fields if not fields_in_ini.get(x)]
        if not_in_ini:
            raise MissingTagsIniFileError(tags=', '.join(not_in_ini),
                                          ini_file=ini_filepath)



from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import user_passes_test


def first_run_required(function=None):
    """
    Decorator for views that are only available if there is no existing user.
    """
    actual_decorator = user_passes_test(
        lambda u: get_user_model().objects.count() == 0,
        login_url='/',
    )
    if function:
        return actual_decorator(function)
    return actual_decorator




# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required as lr

from .views import *

urlpatterns = patterns('',
    url(r'^$', lr(SettingsView.as_view()), name='settings'),
    url(r'^state/$', lr(SetStateSettingsView.as_view()), name='update-state-filter'),
)



# -*- coding: utf-8 -*-
import json
import logging
import traceback
from django.core.exceptions import ObjectDoesNotExist
from django.template.context import RequestContext
import os
from preupg.ui.config.models import AppSettings

from .models import Run, Result
from .forms import *

from django.views.generic import TemplateView, DeleteView, FormView, View
from django.views.generic.list import ListView
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.core.urlresolvers import reverse, reverse_lazy
from preupg.ui.utils.tree import render_result
from preupg.ui.utils.views import is_state_filter, return_error, get_states_to_filter
from django.http.response import Http404
from django.template import loader


logger = logging.getLogger('preup_ui')


class RunsView(ListView):
    template_name = "report/runs.html"
    paginate_by = 25
    context_object_name = 'hostruns'

    def get_queryset(self):
        try:
            query = HostRun.objects.for_result(self.kwargs['result_id'])
        except KeyError:
            query = HostRun.objects.all()
        else:
            return query.select_related('result', 'run', 'host')
        if self.request.GET:
            filter_form = FilterForm(self.request.GET)
            if filter_form.is_valid():
                query = HostRun.objects.finished()
                if filter_form.cleaned_data['hosts']:
                    query = query.by_hosts_processed(filter_form.cleaned_data['hosts'])
                if filter_form.cleaned_data['risk']:
                    query = query.by_risk(filter_form.cleaned_data['risk'])
        query = query.select_related('result', 'run', 'host')
        return query

    def get_action_form(self):
        if self.request.method == 'POST':
            form = ListActionForm(data=self.request.POST)
        else:
            form = ListActionForm()
        form.fields['runs'].choices = [(r.id, r.id) for r in self.get_queryset()]
        return form

    def get_context_data(self, **kwargs):
        context = super(RunsView, self).get_context_data(**kwargs)
        if self.request.GET:
            filter_form = FilterForm(self.request.GET)
        else:
            filter_form = FilterForm()
        context['title'] = 'List of runs'
        context['filter_form'] = filter_form
        context['action_form'] = self.get_action_form()
        return context

    def post(self, request, *args, **kwargs):
        form = self.get_action_form()
        if form.is_valid():
            action = getattr(self, form.cleaned_data['action'])
            return action(form)
        else:
            return self.get(request, *args, **kwargs)

    def delete_selected(self, form):
        # TODO limit number of input values
        hostruns = HostRun.objects.filter(id__in=form.cleaned_data['runs'])
        if form.cleaned_data['confirm']:
            for hostrun in hostruns:
                hostrun.delete()
            return HttpResponseRedirect('{0}?{1}'.format(
                reverse('results-list'),
                self.request.META['QUERY_STRING'],
            ))
        else:
            return TemplateResponse(
                request  = self.request,
                template = 'report/hostrun_delete_selected.html',
                context  = {
                    'form':     form,
                    'hostruns': hostruns,
                },
            )


class DeleteOlderView(FormView):
    template_name   = 'report/hostrun_delete_older.html'
    form_class      = DeleteOlderForm

    def get_form_kwargs(self):
        # ensure that the form is always bound
        return {'data': self.request.REQUEST}

    def get_older_hostruns(self, form):
        hostname = form.is_valid() and form.cleaned_data['host']
        hostruns = HostRun.objects.all()
        if hostname:
            hostruns = hostruns.filter(host__hostname=hostname)
        previous_host_id = None
        for hostrun in hostruns.order_by('host', '-run__dt_submitted'):
            if hostrun.host_id == previous_host_id:
                yield hostrun
            previous_host_id = hostrun.host_id

    def get_context_data(self, **kwargs):
        context = super(DeleteOlderView, self).get_context_data(**kwargs)
        context['hostruns'] = self.get_older_hostruns(context['form'])
        return context

    def form_valid(self, form):
        for hostrun in self.get_older_hostruns(form):
            hostrun.delete()
        return HttpResponseRedirect('{0}?{1}'.format(
            reverse('results-list'),
            self.request.META['QUERY_STRING'],
        ))


class RunView(RunsView):
    template_name = "report/runs.html"

    def get_context_data(self, **kwargs):
        context = super(RunView, self).get_context_data(**kwargs)
        context['title'] = "Run's detail"
        context['filtered_results'] = [int(self.kwargs['result_id'])]
        context['detail'] = True
        context['is_paginated'] = False
        context['url'] = reverse('result-detail', kwargs={'result_id': self.kwargs['result_id']})
        return context


class DeleteRunView(DeleteView):
    model = HostRun
    success_url = reverse_lazy('results-list')
    pk_url_kwarg = 'result_id'


class ResultViewAjax(View):
    """ display specific result, this routine is for ajax"""

    def get(self, request, result_id, *args, **kwargs):
        try:
            result = Result.objects.get(id=result_id)
        except ObjectDoesNotExist:
            return HttpResponse(
                json.dumps({'status': 'ERROR', 'content': "Not found."}),
                content_type='application/json',
            )

        response = {}

        # form with states
        states = get_states_to_filter(request.GET)
        if states:
            form = StateFilterForm(request.GET, result=result)
            try:
                self.request.GET['filter'] == 'all'
            except KeyError:
                pass
            else:
                AppSettings.set_initial_state_filter(states)
        else:
            init_states = AppSettings.get_initial_state_filter()
            inital_conf = {}
            for i in init_states:
                key = i + str(result.id)
                inital_conf[key] = True
            form = StateFilterForm(result=result, initial=inital_conf)

        try:
            search_string = request.GET['search']
        except KeyError:
            search_string = ''

        context = RequestContext(request, {
            'flat_tree': render_result(result, search_string, request.GET),
            'result': result,
            'state_filter_form': form,
        })
        template_name = "report/result.html"
        template = loader.get_template(template_name)
        response['content'] = template.render(context)
        response['status'] = 'OK'
        return HttpResponse(
            json.dumps(response),
            content_type='application/json',
        )


class ReportFilesView(View):
    """ display arbitrary files from scan result """

    def dir_content_html(self, absolute_dir_path, relative_dir_path):
        """Create a simple HTML code containing links to the content of the
        passed directory.
        """
        def sort_dirs_first(filename):
            return (os.path.isfile(os.path.join(absolute_dir_path, filename)),
                    filename.lower())
        sorted_dir_content = sorted(os.listdir(absolute_dir_path),
                                    key=sort_dirs_first)
        content = "<html>\n"
        for filename in sorted_dir_content:
            absolute_filepath = os.path.join(absolute_dir_path, filename)
            file_type_indicator = \
                "DIR  " if os.path.isdir(absolute_filepath) else "FILE "
            relative_filepath = os.path.join(relative_dir_path, filename)
            content += \
                file_type_indicator + "<a href='../file/?path=" + \
                relative_filepath + "'>" + filename + "</a><br>\n"
        content += "</html>\n"
        return content

    def get(self, request, result_id):
        """
        There has to be GET: path='...'
        """
        r = get_object_or_404(Result, id=result_id)
        result_dir = r.get_result_dir()

        try:
            relative_file_path = request.GET['path']
        except KeyError:
            return return_error(request, "Can't open a file because there is"
                                         " no path specified.")

        if not relative_file_path:
            return return_error(request, "Can't open a file because file path"
                                         " is empty.")

        absolute_file_path = os.path.abspath(os.path.join(result_dir,
                                                          relative_file_path))

        if not absolute_file_path.startswith(result_dir):
            return return_error(request, "You are not allowed to access file"
                                         " '%s'." % relative_file_path)

        if os.path.isdir(absolute_file_path):
            response = HttpResponse(self.dir_content_html(absolute_file_path,
                                                          relative_file_path),
                                    mimetype='text/html')
        else:
            try:
                file_content = open(absolute_file_path, 'r')
            except IOError:
                raise Http404('Can\'t open file \'%s\'' % relative_file_path)
            response = HttpResponse(file_content, mimetype='text/plain')
            response["Content-Length"] = os.path.getsize(absolute_file_path)

        return response


class ReportView(View):
    """ display HTML report """
    def get(self, request, result_id):
        r = get_object_or_404(Result, id=result_id)
        file_path = r.get_file_path()

        try:
            f = open(file_path, 'r')
        except IOError:
            raise Http404('Can\'t open HTML report')
        response = HttpResponse(f, mimetype='text/html')

        response["Content-Length"] = os.path.getsize(file_path)

        # this will prompt for download
        #response['Content-Disposition'] = 'attachment; filename=%s' % \
        #    os.path.basename(r.filename)
        return response


class NewHostView(FormView, TemplateView):
    template_name = "report/new_run.html"

    def post(self, request, *args, **kwargs):
        nh_form = NewHostForm(request.POST)
        nr_form = NewRunForm()
        context = {'new_host_form': nh_form}
        context['new_run_form'] = nr_form

        if nh_form.is_valid():
            nh_form.save()
        return self.render_to_response(context)


class NewLocalRunView(FormView, TemplateView):
    template_name = "report/new_run.html"

    def get(self, request, *args, **kwargs):
        #localhost = Host.localhost()
        #run_object = Run.objects.create_for_host(localhost)
        from report.runner import run
        #run(request, run_object)
        try:
            run(request)
        except RuntimeError as ex:
            return return_error(request, str(ex))
        except Exception as ex:
            logger.critical(traceback.format_exc())
            return return_error(request, 'There was an error: \'%s\'' % str(ex))
        return HttpResponseRedirect(reverse('results-list'))


class NewRunView(FormView, TemplateView):
    template_name = "report/new_run.html"

    def post(self, request, *args, **kwargs):
        nh_form = NewHostForm()
        nr_form = NewRunForm(request.POST)
        context = {'new_host_form': nh_form}
        context['new_run_form'] = nr_form

        if nr_form.is_valid():
            run_object = Run.objects.bulk_create_for_run(nr_form.cleaned_data['hosts'])
            from report.runner import run
            run(run_object)
            return HttpResponseRedirect(reverse('results-list'))
        else:
            return self.render_to_response(context)

    def get(self, request, *args, **kwargs):
        context = {
            'new_host_form': NewHostForm(),
            'new_run_form': NewRunForm()
        }
        return self.render_to_response(context)



# -*- coding: utf-8 -*-

from django import forms

from preupg.ui.report.models import Result


class ResultForm(forms.Form):
    left_result = forms.ModelChoiceField(queryset=Result.objects.order_by('-dt_finished'), required=True)
    right_result = forms.ModelChoiceField(queryset=Result.objects.order_by('-dt_finished'), required=True)

    def __init__(self, *args, **kwargs):
        super(ResultForm, self).__init__(*args, **kwargs)

        self.fields['left_result'].widget.attrs['class'] = "selectpicker compare-left-field"
        self.fields['left_result'].widget.attrs['data-live-search'] = "true"
        self.fields['right_result'].widget.attrs['class'] = "selectpicker compare-right-field"
        self.fields['right_result'].widget.attrs['data-live-search'] = "true"



# -*- coding: utf-8 -*-

from django import forms


class UserCreationForm(forms.Form):
    username = forms.RegexField(regex=r'^[\w.@+-]+$', max_length=30, label='Username')
    email = forms.EmailField(label="E-mail", required=False)
    password = forms.CharField(label="Password", required=False)
    password_retyped = forms.CharField(label="Retype Password", required=False)

    def __init__(self, *args, **kwargs):
        self.edit_mode = kwargs.pop('edit_mode', False)
        super(UserCreationForm, self).__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super(UserCreationForm, self).clean()
        password = cleaned_data.get("password")
        email = self.data.get("email", '')
        password_retyped = cleaned_data.get("password_retyped")

        if not email:
            raise forms.ValidationError("Enter an e-mail please.")

        if not self.edit_mode:
            if not password or len(password) <= 0:
                raise forms.ValidationError("Enter a password please.")

            if password != password_retyped:
                raise forms.ValidationError("Passwords do not match.")

        return cleaned_data



import sys
from os.path import dirname, realpath, join

LIB_PATH = join(dirname(realpath(__file__)), "lib")

# add LIB_PATH to PYTHON_PATH
if LIB_PATH not in sys.path:
    sys.path.insert(0, LIB_PATH)



# -*- coding: utf-8 -*-
"""
Distributive execution
"""
try:
    import paramiko
except ImportError:
    pass


def connect(address, username, password):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(address, username=username, password=password)
    return ssh


def get_content():
    return "/root/rpmbuild/BUILD/preupgrade-assistant-0.2.1/contents-users/RHEL5_6/selinux/all-xccdf.xml"


def run_remotely(address, username, password, hostrun_id):
    """
    Execute preupgrade assistant on remote host
    """
    ssh = connect(address, username, password)
    stdin, stdout, stderr = ssh.exec_command("preupg --id %d %s" % (hostrun_id, get_content()))
    for line in stderr.read().splitlines():
        print "'%s'" % line
    for line in stdout.read().splitlines():
        print "'%s'" % line


def run(address, username, password):
    """
    Execute preupgrade assistant on remote host
    """
    ssh = connect(address, username, password)
    stdin, stdout, stderr = ssh.exec_command("preupg.py %s" % get_content())
    for line in stdout.read().splitlines():
        print "'%s'" % line

def main():
    run("192.168.122.209", 'root', 'redhat')


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-

import shutil
import unittest
import tempfile
import preupg

from xml.etree import ElementTree
from preupg.application import Application
from preupg.conf import DummyConf, Conf
from preupg.ui.report.processing import xml_to_html, stringify_children, parse_xml_report
from preupg.ui.report.service import extract_tarball

from django.test import TestCase


class TestXML(TestCase):

    def test_to_html(self):
        links = """\
Prefix <html:a xmlns:html="http://www.w3.org/1999/xhtml/" \
href="./dirtyconf/">dirtyconf/</html:a> suffix. \
Another link <html:a xmlns:html="http://www.w3.org/1999/xhtml/" \
href="./kickstart/noverifycfg/">kickstart/noverifycfg/</html:a> ."""

        b = """\
Prefix <html:b xmlns:html="http://www.w3.org/1999/xhtml/">\
bold text</html:b> suffix."""

        expected_links = """\
Prefix <a href="__INSERT_URL__?path=dirtyconf/" target="_blank">dirtyconf/</a> suffix. \
Another link <a href="__INSERT_URL__?path=kickstart/noverifycfg/" target="_blank">kickstart/noverifycfg/</a> ."""

        expected_b = "Prefix <b>bold text</b> suffix."

        got_links = xml_to_html(links)
        got_b = xml_to_html(b)

        self.assertEqual(got_links, expected_links)
        self.assertEqual(got_b, expected_b)

    def test_stringify_children(self):
        node = ElementTree.fromstring("""<content xmlns:html="http://www.w3.org/1999/xhtml"> \
Text outside tag <html:div>Text <html:em>inside</html:em> tag</html:div> x <b>y</b> z asd\
</content>""")
        r = stringify_children(node).strip()
        expected_r = 'Text outside tag <div>Text <em>inside</em> tag</div> x <b>y</b> z asd'
        self.assertEqual(r, expected_r)

        node2 = ElementTree.fromstring("""<x xmlns:html="http://www.w3.org/1999/xhtml"> \
<html:y>a</html:y></x>""")
        r2 = stringify_children(node2).strip()
        self.assertEqual(r2, '<y>a</y>')

        node3 = ElementTree.fromstring("""<x xmlns:html="http://www.w3.org/1999/xhtml"> a \
<html:y>t<html:y2>a</html:y2>y</html:y>y</x>""")
        r3 = stringify_children(node3).strip()
        self.assertEqual(r3, 'a <y>t<y2>a</y2>y</y>y')


# class TestImport(TestCase):
#     def setUp(self):
#         self.temp_dir = tempfile.mkdtemp()
#         conf = {
#             "contents": "tests/RHEL6_7/dummy_preupg/all-xccdf.xml",
#             "profile": "xccdf_preupg_profile_default",
#             "result_dir": self.temp_dir,
#             "skip_common": True,
#             "temp_dir": self.temp_dir,
#             "debug": True,  # so root check won't fail
#         }
#         self.conf = DummyConf(**conf)
#
#     def tearDown(self):
#         shutil.rmtree(self.temp_dir)
#
#     def test_report_import(self):
#         """
#         "scan" system with dummy content and import it
#         """
#         a = Application(Conf(self.conf, preupg.settings))
#         tarball_path = a.scan_system()
#
#         xml_path, html_path = extract_tarball(tarball_path, self.temp_dir)
#         self.assertTrue(len(parse_xml_report(xml_path)) > 0)


if __name__ == '__main__':
    unittest.main()




from __future__ import unicode_literals
import datetime
import os
import subprocess
from preupg.logger import settings, logger_report, log_message, logging


class ScanningHelper(object):

    @staticmethod
    def compare_data(row):
        """Function sorts a data output"""
        test_cases = {'error': '01',
                      'fail': '02',
                      'needs_action': '03',
                      'needs_inspection': '04',
                      'fixed': '05',
                      'informational': '06',
                      'pass': '07',
                      'notapplicable': '08',
                      'notchecked': '09'}
        try:
            dummy_title, dummy_rule_id, result = row.split(':')
        except ValueError:
            return '99'
        else:
            try:
                return test_cases[result]
            except KeyError:
                return '99'

    @staticmethod
    def format_rules_to_table(output_data, content):
        """Function format output_data to table"""
        if not output_data:
            # If output_data does not contain anything then do not print nothing
            return
        max_title_length = max(x for x in [len(l.split(':')[0]) for l in output_data]) + 5
        max_result_length = max(x for x in [len(l.split(':')[2]) for l in output_data]) + 2
        log_message(settings.result_text.format(content))
        message = '-' * (max_title_length + max_result_length + 4)
        log_message(message)
        for data in sorted(output_data, key=ScanningHelper.compare_data, reverse=True):
            try:
                title, dummy_rule_id, result = data.split(':')
            except ValueError:
                # data is not an information about processed test; let's log it as an error
                log_message(data, level=logging.ERROR)
            else:
                log_message(u"|%s |%s|" % (title.ljust(max_title_length),
                                          result.strip().ljust(max_result_length)))
        log_message(message)


class ScanProgress(object):
    """The class is used for showing progress during the scan check."""
    def __init__(self, total_count, debug):
        self.total_count = total_count
        self.current_count = 0
        self.output_data = []
        self.debug = debug
        self.names = {}
        self.list_names = []
        self.width_size = 0
        self.time = datetime.datetime.now()

    def get_full_name(self, count):
        """Function returns full name from dictionary"""
        try:
            key = self.list_names[count]
        except IndexError:
            return ''
        return self.names[key]

    @staticmethod
    def get_terminal_width():
        """
        Function returns terminal width

        :return:
        """
        cmd = ["stty", "size"]
        sp = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return int(sp.communicate()[0].split()[1])

    def _return_correct_msg(self, msg):
        if len(msg) > self.width_size:
            msg = msg[:self.width_size - 7] + '...'
        return msg

    def show_progress(self, stdout_data):
        """Function shows a progress of assessment"""
        try:
            self.width_size = ScanProgress.get_terminal_width()
        except IndexError:
            self.width_size = 80
        logger_report.debug(stdout_data.strip())
        xccdf_rule = ""
        dummy_result = ""
        try:
            xccdf_rule, dummy_result = stdout_data.strip().split(':')
        except ValueError:
            print (stdout_data)
            return
        self.output_data.append(u'{0}:{1}'.format(self.names[xccdf_rule], stdout_data.strip()))
        self.current_count += 1
        old_width = self.width_size
        self.width_size -= 21
        prev_msg = self._return_correct_msg(self.get_full_name(self.current_count - 1))
        self.width_size = old_width
        cur_msg = self._return_correct_msg(self.get_full_name(self.current_count))
        cnt_back = 7 + len(prev_msg) + 3
        curr_time = datetime.datetime.now()
        diff_time = curr_time - self.time
        msg = (u'%sdone    (%s) (time: %.2d:%.2ds)'
               % ('\b' * cnt_back,
                  prev_msg,
                  diff_time.seconds / 60,
                  diff_time.seconds % 60))
        log_message(msg, new_line=True)
        if self.total_count > self.current_count:
            msg = self._return_correct_msg(u'%.3d/%.3d ...running (%s)'
                                           % (self.current_count + 1,
                                              self.total_count,
                                              cur_msg))
            log_message(msg, new_line=False)
        self.time = datetime.datetime.now()

    def set_names(self, names):
        """
        Function sets names of each rule

        names have format:
                key= xccdf_preupg_...
                value = "Full description"
        """
        self.names = names
        self.list_names = sorted(names)

    def get_output_data(self):
        """Function gets an output data from oscap"""
        return self.output_data

    def update_data(self, changed_fields):
        """Function updates a data"""
        for index, row in enumerate(self.output_data):
            try:
                title, rule_id, dummy_result = row.split(':')
                logger_report.debug(row)
            except ValueError:
                continue
            else:
                result_list = [x for x in changed_fields if rule_id in x]
                if result_list:
                    self.output_data[index] = u"%s:%s:%s" % (title, rule_id, result_list[0].split(':')[1])




class MissingHeaderCheckScriptError(RuntimeError):
    """
    Error indicating problems with missing header in check script.
    """
    def __init__(self, script=""):
        if script:
            message = "A header missing in the %s check script" % script
        else:
            message = "A header missing in a check script"
        RuntimeError.__init__(self, "Error: " + message)


class ModuleSetFormatError(RuntimeError):
    """
    Module set has wrong format

    A mandatory part is missing, has wrong mode, etc.
    """
    def __init__(self, reason, subject="", e=None):
        lines = ["Module set has wrong format:"]
        lines.append("    reason: %s" % reason)
        if subject:
            lines.append("    subject: %s" % subject)
        if e is not None:
            lines.append("    error: %s" % e)
        RuntimeError.__init__(self, "Error: " + '\n'.join(lines))


class ModuleSetInitError(RuntimeError):
    """
    Module set init script failed
    """
    def __init__(self, returncode, stderr=""):
        lines = ["Module set init script reported error:"]
        if returncode:
            lines.append("    status: %s" % returncode)
        if stderr:
            for line in stderr.strip('\n').split('\n'):
                lines.append("    stderr: %s" % line)
        RuntimeError.__init__(self, "Error: " + '\n'.join(lines))


class MissingFileInContentError(RuntimeError):
    """
    Some file is missing in a module.
    """
    def __init__(self, file="", dir=""):
        if file and dir:
            message = "File %s missing in the module directory %s" \
                      % (file, dir)
        else:
            message = "File(s) missing in a module"
        RuntimeError.__init__(self, "Error: " + message)


class MissingTagsIniFileError(RuntimeError):
    """
    Some tags are missing in INI file.
    """
    def __init__(self, tags="", ini_file=""):
        if tags and ini_file:
            message = "Tag(s) %s missing in the %s file" % (tags, ini_file)
        else:
            message = "Tag(s) missing in an INI file"
        RuntimeError.__init__(self, "Error: " + message)


class EmptyTagGroupXMLError(ValueError):
    """
    One of the tags in group.xml (to be merged into all-xccdf.xml for
    OpenSCAP) is empty.
    """
    def __init__(self, tag=""):
        if tag:
            message = "Tag %s in a group.xml can't be empty" % tag
        else:
            message = "A tag in a group.xml can't be empty"
        ValueError.__init__(self, "Error: " + message)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

"""
Class creates a kickstart for migration scenario
"""

import base64
import shutil
import os
import imp

from pykickstart.constants import KS_MISSING_IGNORE, KS_SCRIPT_POST
from pykickstart.parser import KickstartParser, KickstartError, Script
from pykickstart.version import makeVersion

from preupg.logger import logger, logging, LoggerHelper, logger_debug
from preupg.logger import log_message
from preupg.utils import FileHelper, ProcessHelper, MessageHelper
from preupg import settings


class BaseKickstart(object):
    """ Base class used for testing tool run on final pkgs. """

    def run_module(self, *arg, **kwargs):
        """Perform the check itself and return results."""
        raise NotImplementedError()


class KickstartGenerator(object):
    """Generate kickstart using data from provided result"""

    def __init__(self, conf, dir_name, kickstart_name):
        self.dir_name = dir_name
        self.ks = None
        self.kickstart_name = kickstart_name
        self.ks_list = []
        self.repos = None
        self.latest_tarball = ""
        self.temp_file = '/tmp/part-include'
        self.conf = conf
        self.groups = []
        self.packages = []
        self.part_layout = None
        self._injector_type = 'BaseKickstart'
        self.plugin_classes = {}
        self._add_debug_log_file()

    def _add_debug_log_file(self):
        """
        Add the special report log file
        :return:
        """
        try:
            LoggerHelper.add_file_handler(logger_debug,
                                          settings.preupg_log,
                                          formatter=logging.Formatter("%(asctime)s %(levelname)s\t%(filename)s"
                                                            ":%(lineno)s %(funcName)s: %(message)s"),
                                          level=logging.DEBUG)
        except (IOError, OSError):
            logger.warning("Can not create debug log '%s'", settings.preupg_log)
        else:
            self.debug_log_file = settings.preupg_log

    def checker_find_injector(self, module):
        injectors = []
        for n in dir(module):
            attr = getattr(module, n)
            if hasattr(attr, '__base__') and attr.__base__.__name__ == self._injector_type:
                injectors.append(attr)
        return injectors

    def load_plugins(self, dir_name):
        plugin_checkers = {}
        plugins_dir = os.path.join(dir_name, 'plugins')
        for plugin in os.listdir(plugins_dir):
            if not plugin.endswith('.py'):
                continue
            modname, suffix = plugin.rsplit('.', 1)
            if suffix == 'py':
                fullpath = os.path.abspath(plugins_dir)
                f, filename, description = imp.find_module(modname, [fullpath])
                m = imp.load_module(modname, open(filename, 'U'), filename, description)
                try:
                    injs = self.checker_find_injector(m)
                    for i in injs:
                        obj = i(self.ks.handler)
                        plugin_checkers[modname] = obj
                except AttributeError as ae:
                    print (ae)
                    print ("Module '%s' does not implement `register(context)`" % modname)
        return plugin_checkers

    def get_supported_modules(self):
        """Return list of supported tools"""
        return self.plugin_classes.keys()

    def collect_data(self):
        self._remove_obsolete_data()
        collected_data = True
        self.ks = KickstartGenerator.load_or_default(
            KickstartGenerator.get_kickstart_path(self.dir_name),
            os.path.join(self.dir_name, settings.KS_TEMPLATE))
        if self.ks is None:
            collected_data = False
        self.latest_tarball = self.get_latest_tarball()
        return collected_data

    def _remove_obsolete_data(self):
        if os.path.exists(KickstartGenerator.get_kickstart_path(self.dir_name)):
            lines = FileHelper.get_file_content(KickstartGenerator.get_kickstart_path(self.dir_name), "r", method=True)
            lines = [x for x in lines if not x.startswith('key')]
            FileHelper.write_to_file(KickstartGenerator.get_kickstart_path(self.dir_name), "w", lines)

    @staticmethod
    def get_kickstart_path(dir_name):
        return os.path.join(dir_name, 'anaconda-ks.cfg')

    @staticmethod
    def load_or_default(system_ks_path, ks_template):
        """ load system ks or default ks """
        ksparser = KickstartParser(makeVersion())
        try:
            ksparser.readKickstart(system_ks_path)
        except (KickstartError, IOError):
            log_message("Cannot read the system Kickstart at %s." % system_ks_path)
            try:
                ksparser.readKickstart(ks_template)
            except AttributeError:
                log_message("There is no KS_POSTSCRIPT_TEMPLATE specified in settings.py.", level=logging.DEBUG)
            except IOError:
                log_message("Cannot read the Kickstart template %s." % ks_template)
                return None
        return ksparser

    def set_autopart(self):
        """Letting kickstart to do the partitioning automatically is the safe
        option (as opposed to guessing the correct partitioning parameters for
        the new system).
        """
        self.ks.handler.autopart(autopart=True)

    def delete_obsolete_issues(self):
        """ Remove obsolete items which does not exist on RHEL-7 anymore"""
        self.ks.handler.bootloader.location = None

    def mitigate_security_issues(self):
        """It may pose a security risk to put the root password into the
        kickstart, even if it is a hash because weak hash algorithm could have
        been used.
        Also, set the default hashing algorithm for user passwords to the
        strong sha-512.
        """
        self.ks.handler.rootpw(isCrypted=False,
                               password="<password_placeholder>")
        self.ks.handler.authconfig(
            authconfig="--enableshadow --passalgo=sha512")

    def embed_script(self, tarball):
        if tarball is None:
            return
        tarball_content = None
        if os.path.exists(tarball):
            tarball_content = FileHelper.get_file_content(tarball, 'rb', decode_flag=False)
            tarball_name = os.path.splitext(os.path.splitext(os.path.basename(tarball))[0])[0]
        script_str = ''
        try:
            script_path = settings.KS_POSTSCRIPT_TEMPLATE
        except AttributeError:
            log_message('KS_POSTSCRIPT_TEMPLATE is not defined in settings.py.')
            return
        script_str = FileHelper.get_file_content(os.path.join(settings.KS_DIR, script_path), 'rb')
        if not script_str:
            log_message("Cannot open the script template: {0}.".format(script_path))
            return
        if tarball_content is not None:
            script_str = script_str.replace('{tar_ball}', base64.b64encode(tarball_content))
            script_str = script_str.replace('{RESULT_NAME}', tarball_name)
            script_str = script_str.replace('{TEMPORARY_PREUPG_DIR}', '/root/preupgrade')
            script = Script(script_str, type=KS_SCRIPT_POST, inChroot=True)
            self.ks.handler.scripts.append(script)

    def save_kickstart(self):
        kickstart_data = self.ks.handler.__str__()
        FileHelper.write_to_file(self.kickstart_name, 'wb', kickstart_data)

    def update_kickstart(self, text, cnt):
        self.ks_list.insert(cnt, text)
        return cnt + 1

    @staticmethod
    def copy_kickstart_templates():
        # Copy kickstart files for kickstart generation
        for file_name in settings.KS_TEMPLATES:
            target_name = os.path.join(settings.KS_DIR, file_name)
            source_name = os.path.join(settings.data_dir, 'templates',
                                       file_name)
            if not os.path.exists(target_name) and os.path.exists(source_name):
                try:
                    shutil.copy(source_name, target_name)
                except IOError:
                    pass

    def get_prefix(self):
        return settings.tarball_prefix + settings.tarball_base

    def get_latest_tarball(self):
        tarball = None
        for directories, dummy_subdir, filenames in os.walk(settings.tarball_result_dir):
            preupg_files = [x for x in sorted(filenames) if x.startswith(self.get_prefix())]
            # We need a last file
            tarball = os.path.join(directories, preupg_files[-1])
        return tarball

    def comment_kickstart_issues(self):
        """Comment out those line in kickstart that are to be reviewed
        by user and uncommented by themself if necessary.
        """
        list_issues = [' --', 'group', 'user ', 'repo', 'url', 'rootpw',
                       'part', 'volgroup', 'logvol', 'raid']
        kickstart_data = []
        try:
            kickstart_data = FileHelper.get_file_content(os.path.join(settings.KS_DIR, self.kickstart_name),
                                                         'rb',
                                                         method=True,
                                                         decode_flag=False)
        except IOError:
            log_message("The %s file is missing. The partitioning layout might not be complete." % self.kickstart_name,
                        level=logging.WARNING)
            return None
        for index, row in enumerate(kickstart_data):
            tag = [com for com in list_issues if row.startswith(com)]
            if tag:
                kickstart_data[index] = "#" + row
        FileHelper.write_to_file(self.kickstart_name, 'wb', kickstart_data)

    def generate(self):
        if not self.collect_data():
            log_message("Important data are missing for the Kickstart generation.", level=logging.ERROR)
            return None
        self.ks.handler.packages.excludedList = []
        self.plugin_classes = self.load_plugins(os.path.dirname(__file__))
        for module in iter(self.plugin_classes.keys()):
            self.plugin_classes[module].run_module()
        self.ks.handler.packages.handleMissing = KS_MISSING_IGNORE
        self.ks.handler.keyboard.keyboard = 'us'
        self.embed_script(self.latest_tarball)
        self.set_autopart()
        self.delete_obsolete_issues()
        self.mitigate_security_issues()
        self.save_kickstart()
        self.comment_kickstart_issues()
        return True

    def main(self):
        if not os.path.exists(os.path.join(settings.assessment_results_dir,
                                           settings.xml_result_name)):
            log_message("The 'preupg' command was not run yet. Run it before the Kickstart generation.")
            return 1

        KickstartGenerator.copy_kickstart_templates()
        dummy_ks = self.generate()
        if dummy_ks:

            tar_ball_dir = os.path.basename(self.latest_tarball).split('.')[0]
            kickstart_dir = os.path.join(os.path.dirname(self.dir_name),
                                         tar_ball_dir)
            log_message(settings.kickstart_text % (self.kickstart_name,
                                                   kickstart_dir,
                                                   kickstart_dir))
        KickstartGenerator.kickstart_scripts()

    @staticmethod
    def kickstart_scripts():
        ks_scripts_file = os.path.join(settings.data_dir, "preassessment",
                                       settings.KS_SCRIPTS)
        try:
            lines = FileHelper.get_file_content(ks_scripts_file, "rb", True)
            for line in lines:
                line = line.strip()
                if line.startswith("#"):
                    continue
                if 'is not installed' in line:
                    continue
                cmd, name = line.split("=", 2)
                kickstart_file = os.path.join(settings.KS_DIR, name)
                ProcessHelper.run_subprocess(cmd, output=kickstart_file, shell=True)
        except IOError:
            pass


def run():
    KickstartGenerator()
    return



# -*- coding: utf-8 -*-

from django.utils.datastructures import SortedDict
from preupg.ui.config.models import AppSettings
from preupg.ui.report.models import TestResult, TestGroupResult
from .views import get_states_to_filter


def get_groups_children(group, groups):
    """
    we want to operate on the same set of elements, so we can use caching
    """
    return [x for x in groups if x.parent and x.parent.id == group.id]


def get_groups_tests(group, tests):
    """
    we want to operate on the same set of elements, so we can use caching
    """
    return [x for x in tests if x.group.id == group.id]


class RecursiveFilter(object):
    """
    go through graph structure of groups and filter requested group.
    This creates new attributes on groups:
     \ filtered_tests -- filtered tests in this category only
     \ allfiltered_tests -- tests in this category and its children
    """
    def __init__(self, groups, tests):
        """ groups, tests are queries """
        self.groups = groups
        self.tests = tests
        self.test_ids = tests.values_list('id', flat=True)

    def testresults_filter(self, group):
        """ self.tests is already performed filter, lets grep tests matching provided group """
        if hasattr(group, 'filtered_tests'):
            return group.filtered_tests[:]
        groups_tests = get_groups_tests(group, self.tests)
        group.filtered_tests = []
        for groups_test in groups_tests:
            # django's lazy queries won't evaluate self.test_ids first time
            # if groups_test.id in self.test_ids:
            if groups_test.id in iter(self.test_ids):
                group.filtered_tests.append(groups_test)
        return group.filtered_tests[:]

    def subelements_filter(self, group):
        """
        recursively traverse to group's children and return all tests which will be displayed
        """
        if hasattr(group, 'allfiltered_tests'):
            return group.allfiltered_tests[:]

        allfiltered_tests = self.testresults_filter(group)

        for child in get_groups_children(group, self.groups):
            # and do the same for children
            allfiltered_tests += self.subelements_filter(child)
        group.allfiltered_tests = allfiltered_tests
        return allfiltered_tests[:]


class RecursiveRenderer(object):
    def __init__(self, groups_dict):
        """
        {
            'group': {
                'group': None,
                'group': None,
            },
            'group': None,
        }
        """
        self.groups = groups_dict
        self.result = []

    def _append_group(self, group):
        self.result.append(('GROUP', group))

    def _append_tag(self, tag):
        self.result.append(('TAG', tag))

    def _append_tests(self, group):
        """ in template, we use group.filtered_tests """
        self.result.append(('TESTS', group))

    def _append_group_with_tests(self, group):
        """ group has no subgroups, lets display it with tests at once """
        self.result.append(('GROUP_WITH_TESTS', group))

    def translate_groups(self, groups):
        """
        This recursive function renders this:
        <ul root-groups>
          <li>
            <group>
              <ul>
                <li>
                  <group>
                    <ul>
                      <tests>
                    </ul>
                </li>
                <li>
                  <group>
                    <ul>
                      <tests>
                    </ul>
                </li>
                <tests>
              </ul>
            <group>
          </li>
        </ul>
        """
        for group, children in groups.iteritems():
            # starting li tags for groups are in template, so the tag can be tweaked
            if children is None:
                # there are no children, display group with ite tests
                self._append_group_with_tests(group)
            else:
                # this is more complex scenario:
                # this group has subgroups and may have tests
                self._append_group(group)
                self._append_tag('<ul class="entry-list">')
                # lets display recursively children
                self.translate_groups(children)
                # subgroups precede tests of current group
                if group.filtered_tests:
                    self._append_tests(group)
                self._append_tag('</ul>')
            self._append_tag('</li>')

    def render(self):
        """
        return a list of tuples where first item of tuple is
        information of what is in second place:
            ({TAG,GROUP,TESTS}, <object>)
        """
        self.translate_groups(self.groups)
        return self.result


def render_result(result, search_string=None, get=None):
    filtering_active = bool(search_string or get)

    # convert states to those in DB
    state_filter = TestResult.TEST_STATES.list_keys(get_states_to_filter(get))
    if not state_filter:
        state_filter = TestResult.TEST_STATES.list_keys(AppSettings.get_initial_state_filter())

    # perform global queries in tests and groups; we'll use our caching for these
    groups = TestGroupResult.objects.for_result(result)
    root_groups = groups.root()
    tests = TestResult.objects.for_result(result).by_states(state_filter)
    if search_string:
        groups = groups.search_in_title(search_string)
        tests = tests.search_in_title(search_string)
    groups = groups.select_related('parent', 'group', 'group__parent')
    tests = tests.select_related().prefetch_related('testlog_set', 'risk_set')

    rf = RecursiveFilter(groups, tests)

    def is_requested(group, force=False):
        """
        lets put together the tree structure and check if there is anything to display in this group
        """
        count = len(rf.subelements_filter(group))

        if count > 0 or force:
            group.displayed_count = count
            return True
        return False

    def filter_children(children, level):
        """ recursively go through provided groups """
        ch_dict = SortedDict()
        for child in children:
            child.left_margin = level + 1
            child.child_left_margin = level + 2
            is_child_requested = is_requested(child, level == -1 and not filtering_active)

            if is_child_requested:
                groups_gchildren = get_groups_children(child, groups)
                if len(groups_gchildren) > 0:
                    ch_dict[child] = filter_children(groups_gchildren, level + 1)
                else:
                    ch_dict[child] = None
        return ch_dict

    if not filtering_active:
        root_groups = filter(lambda x: x.parent is None, groups)

    groups_dict = filter_children(root_groups, -1)

    result_list = RecursiveRenderer(groups_dict).render()

    return result_list



# -*- coding: utf-8 -*-

"""
Class creates a kickstart for migration scenario
"""

import os

from pykickstart.constants import CLEARPART_TYPE_ALL
from preupg.kickstart.application import BaseKickstart
from preupg.utils import FileHelper
from preupg.logger import log_message, logging
from preupg import settings


class PartitionGenerator(BaseKickstart):
    """Generate partition layout"""
    def __init__(self, handler):
        self.part_layout = None
        self.layout = None
        self.vg_info = None
        self.lvdisplay = None
        self.raid_devices = {}
        self.vol_group = {}
        self.logvol = {}
        self.part_dict = {}
        self.handler = handler
        self.parts = []
        self.vg_list = []
        self.lv_list = []
        self.raid_list = []

    def generate_partitioning(self):
        pv_name = ""
        index_pv = 1
        crypt = ""
        if self.layout is None:
            return None
        for index, row in enumerate(self.layout):
            fields = row.strip().split(' ')
            device = fields[0]
            size = fields[3]
            multiple = 1
            if size.endswith('G'):
                multiple = 1000
                # Converting to MB from GB
            size = int(float(size[:-1])) * multiple
            device_type = fields[5]
            try:
                mount = fields[6]
                if mount == '[SWAP]':
                    mount = 'swap'
            except IndexError:
                mount = None
            if device_type == 'disk' or device_type == 'crypt' or \
                    device_type == 'rom':
                continue
            if device_type == 'part':
                if not mount:
                    ident = index_pv
                    pv_name = 'pv.%.2d' % int(ident)
                    try:
                        new_row = self.layout[index + 1].strip()
                        if 'raid' in new_row:
                            continue
                        if 'part' in new_row:
                            continue
                        new_row_fields = new_row.split()
                        if 'crypt' in new_row_fields:
                            crypt = ' --encrypted'
                            try:
                                pv_name = new_row_fields[6]
                            except IndexError:
                                pass
                    except IndexError:
                        pass
                    if pv_name not in self.part_dict:
                        self.part_dict[pv_name] = {}
                    self.part_dict[pv_name]['size'] = size
                    self.part_dict[pv_name]['crypt'] = crypt
                    crypt = ""
                    index_pv += 1
                    continue
                else:
                    device = ''.join([x for x in device if not x.isdigit()])
                    if mount not in self.part_dict:
                        self.part_dict[mount] = {}
                    self.part_dict[mount]['size'] = size
                    self.part_dict[mount]['device'] = device
                    self.part_dict[mount]['crypt'] = ""
                    continue
            if 'raid' in device_type:
                raid_type = device_type[-1]
                try:
                    new_row_fields = self.layout[index + 1].strip().split()
                    if 'crypt' in new_row_fields:
                        crypt = ' --encrypted --passphrase='
                        fields = self.layout[index + 1].strip().split()
                        mount = fields[6]
                except IndexError:
                    pass
                if mount not in self.raid_devices:
                    self.raid_devices[mount] = {}
                    self.raid_devices[mount]['raid_devices'] = []
                self.raid_devices[mount]['raid_devices'].append(index_pv)
                self.raid_devices[mount]['level'] = raid_type
                self.raid_devices[mount]['crypt'] = crypt
                crypt = ""
                index_pv += 1
                continue
            if device_type == 'lvm':
                if self.vg_info is None or not self.vg_info:
                    continue
                try:
                    vg_name = [x for x in iter(self.vg_info.keys())
                               if device.startswith(x)][0]
                except IndexError:
                    return
                # Get volume group name
                if vg_name not in self.vol_group:
                    self.vol_group[vg_name] = {}
                self.vol_group[vg_name]['pesize'] = 4096
                self.vol_group[vg_name]['pv_name'] = pv_name
                if self.lvdisplay is None or not self.lvdisplay:
                    continue
                lv_name = [x for x in iter(self.lvdisplay.keys())
                           if x in device][0]
                if mount not in self.logvol:
                    self.logvol[mount] = {}
                self.logvol[mount]['vgname'] = vg_name
                self.logvol[mount]['size'] = size
                self.logvol[mount]['lv_name'] = lv_name

    def _get_part_devices(self):
        for key, value in sorted(iter(self.part_dict.items())):
            if value['crypt'] == "":
                try:
                    self.parts.append(self.handler.PartData(
                        size=value['size'], mountpoint=key,
                        disk=value['device'])
                    )
                except KeyError:
                    self.parts.append(self.handler.PartData(size=value['size'],
                                                            mountpoint=key))
            else:
                self.parts.append(self.handler.PartData(
                    size=value['size'], mountpoint=key,
                    encrypted=value['crypt'])
                )

    def _get_logvol_device(self):
        for key, value in sorted(iter(self.logvol.items())):
            self.lv_list.append(self.handler.LogVolData(
                name=value['lv_name'], vgname=value['vgname'],
                size=value['size'], mountpoint=key)
            )

    def _get_vg_device(self):
        for key, value in iter(self.vol_group.items()):
            pv_name = value['pv_name']
            self.vg_list.append(self.handler.VolGroupData(
                vgname=key, physvols=[pv_name], pesize=value['pesize'])
            )

    def _get_raid_devices(self):
        for key, value in iter(self.raid_devices.items()):
            level = value['level']
            members = []
            for index in value['raid_devices']:
                member = "raid.%.5d" % int(index)
                members.append(member)
                self.parts.append(self.handler.PartData(grow=True, size=2048,
                                                        mountpoint=member))
            device = "md%s" % level
            self.raid_list.append(self.handler.RaidData(
                level=level, mountpoint=key, device=device,
                members=members, encrypted=value['crypt'])
            )

    @staticmethod
    def get_volume_info(filename, first_index, second_index):
        try:
            volume_list = FileHelper.get_file_content(filename, 'rb',
                                                      method=True,
                                                      decode_flag=False)
        except IOError:
            log_message("The %s file is missing. The partitioning layout might"
                        " not be complete." % filename, level=logging.WARNING)
            return None
        volume_info = {}
        for line in volume_list:
            fields = line.strip().split(':')
            volume_info[fields[first_index]] = fields[second_index]
        return volume_info

    def get_partition_layout(self, lsblk, vgs, lvdisplay):
        lsblk_filename = os.path.join(settings.KS_DIR, lsblk)
        try:
            self.layout = FileHelper.get_file_content(lsblk_filename, 'rb',
                                                      method=True,
                                                      decode_flag=False)
        except IOError:
            log_message("The %s file was not generated by the module. "
                        "Kickstart does not contain the partitioning layout"
                        % lsblk_filename)
            self.part_layout = None
            return None
        if vgs is not None:
            self.vg_info = PartitionGenerator.get_volume_info(
                os.path.join(settings.KS_DIR, vgs), 0, 5
            )
        if lvdisplay is not None:
            self.lvdisplay = PartitionGenerator.get_volume_info(
                os.path.join(settings.KS_DIR, lvdisplay), 0, 1
            )

    def get_partitioning(self):
        self.handler.clearpart.type = CLEARPART_TYPE_ALL
        self._get_part_devices()
        self._get_vg_device()
        self._get_logvol_device()
        self._get_raid_devices()
        self.handler.partition(partitions=self.parts)
        self.handler.logvol(lvList=self.lv_list)
        self.handler.volgroup(vgList=self.vg_list)
        self.handler.raid(raidList=self.raid_list)

    def run_module(self, *args, **kwargs):
        self.get_partition_layout('lsblk_list', 'vgs_list', 'lvdisplay')
        self.generate_partitioning()
        self.get_partitioning()
